﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.IO;
using System.Runtime.CompilerServices;

namespace EgMaint
{

    /// <summary>
    /// 定義
    /// </summary>
    public static partial class CDefine
    {
        //エラーコード
        public const uint SUCCESS = 0x00000000;       //成功

        public const uint ERR_NOT_START_PROCESS = 0x00000009; //プロセスの起動に失敗

        public const uint ERR_GATHER_NOTTARGETFILE = 0x0011; //収集機能用ファイルが登録されていない
        public const uint ERR_GATHER_FATAL_TARGETFILE = 0x0012; //収集機能用ファイルパスが正しくない
        public const uint ERR_GATHER_EXIST_TARGETFILE = 0x0013; //収集機能用ファイルパスが存在しないパスが指定されている
        public const uint ERR_DLG_NOTTARGETTAG = 0x0013;     //診断用TagRefOutが登録されていない
        public const uint ERR_RETAIN_NOTTARGETTAG = 0x0021;  //リテイン用TagRefが登録されていない
        public const uint ERR_RETAIN_NOTTARGETTAG2 = 0x0022; //リテイン用TagRefが登録されていたが、システム上エラーが起きている
        public const uint ERR_RETAIN_READLIST = 0x0023;      //リテインファイルが存在しない、またはTag情報がありません
        public const uint ERR_RETAIN_WRITETAG = 0x0024;      //リテイン反映時Tag反映に失敗

        public const uint ERR_FATAL_ERROR = 0xffffffff;      //システムエラー

        //try-catch エラー
        public const uint ERR_TRY_CATCH = 0x00010000;           //try-catchでのエラー

        public const int INIT_STS_IDLE = 0;                     // サービス未初期化
        public const int INIT_STS_INIT = 1;                     // サービス初期化中
        public const int INIT_STS_INITRUN = 2;                  // サービス初期化中(RUN受信済み)
        public const int INIT_STS_INITEND = 3;                  // サービス初期化完了
    }

    public class CommonFunc
    {
        /// <summary>
        /// 実行中の行数取得(呼び出し元の行番号返却)
        /// </summary>
        /// <param name="line"></param>
        /// <returns></returns>
        public static UInt16 GetLine([CallerLineNumber]int line = 0)
        {
            ushort uiLine;

            if (UInt16.TryParse(line.ToString(), out uiLine) == false)
            {
                uiLine = 0;
            }
            return uiLine;
        }

        /// <summary>
        /// 実行中のファイルネーム取得(呼び出し元のファイルパスを取得し、ファイルネームを返却)
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public static string GetFileName([CallerFilePath]string path = "")
        {
            return Path.GetFileName(path);
        }

        /// <summary>
        /// ﾌｫﾙﾀﾞが存在しているかﾁｪｯｸし、なければﾌｫﾙﾀﾞを作成する
        /// </summary>
        /// <param name="srPath"></param>
        /// <returns></returns>
        public static bool gfMakeFolder(string srPath)
        {
            // フォルダが存在しているかチェックする
            if (System.IO.Directory.Exists(srPath))
            {                                                           // 既にフォルダが存在している場合
                return true;                                            // 成功を返して処理を終了する
            }

            try
            {
                System.IO.Directory.CreateDirectory(srPath);            // フォルダを生成する
            }
            catch (Exception)
            {                                                           // 例外異常が発生した場合
                return false;                                           // 失敗を返して処理を終了する
            }
            return true;
        }
        /// <summary>
        /// パスの整合性チェック
        /// </summary>
        /// <param name="srPath">パス</param>
        /// <param name="srMsg">メッセージ</param>
        /// <returns>false=無効なパス / true=有効なパス</returns>
        public static bool fnIsValidPath(ref string srPath)
        {
            bool boRet;
            try
            {
                // フルパスの取得
                srPath = Path.GetFullPath(srPath);
                // フルパス取得成功のため、有効なパスと判断
                boRet = true;
            }
            catch
            {
                // 例外発生のため、無効なパスと判断
                boRet = false;
            }

            return boRet;
        }
        /// <summary>
        /// csvの要素の前後に""をつける
        /// </summary>
        /// <typeparam name="T"></typeparam>
        public class CsvList<T> : System.Collections.Generic.List<string>
        {
            // Listt<T>のAddを隠蔽し再定義（orverrideできなため）
            public new void Add(string item)
            {
                string newStr = item.Replace("\"", "\"\"");
                // ダブルクォーテーションで文字列を囲む
                base.Add("\"" + newStr + "\"");
            }
        }
    }

}
