﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using RTedge;

namespace EgMDiag
{
    /// <summary>
    /// RT-Edgeの操作クラス
    /// </summary>
    public partial class EdgeService
    {
        private static EdgeService service = new EdgeService(); // EdgeServiceクラス生成
        MainWindow _Main = null;                                // 親クラス
        Diag diag = null;                                       // 診断クラス
        /// <summary>
        /// RT-edge container common
        /// </summary>
        public edge_API EGAPI;                                              // edge APIのインスタンス
        public string ServiceRealName = "";                                 // xmlで指定したサービスコンテナ名 EgInit後設定
        public string DefServiceName = "EgMDiag";                           // サービスコンテナ名デフォルト値 EgInit前で使用
        public ServiceLiveIndicatorTimer ServiceIndLiveTimer;               // サービスインジケータ用のタイマ
        public byte InitState = CDefine.INIT_STS_IDLE;                      // サービス初期化状態(0:未初期化, 1:初期化中, 2:初期化中(Run受信済み), 3:初期化完了)

        public const SByte DIAG_THMODE_NOTDEFINE = -1;//定義されていない
        public const SByte DIAG_THMODE_SMALL = 0;// target < TH = err
        public const SByte DIAG_THMODE_BIG = 1;// target > TH = err

        public const SByte DIAG_THRESULT_NOTCHECK = -1; //未実施
        public const SByte DIAG_THRESULT_ERR = 0;       //ERR
        public const SByte DIAG_THRESULT_OK = 1;        //OK

        public bool bFirst_SetTagKind_Flag = false;      //Tagの種類区別作成フラグ
        /// <summary>
        /// 入出力スレッド情報
        /// </summary>
        public class IO_TRDINFO
        {
            public SemaphoreSlim hTrigSem;              // 入出力トリガセマフォハンドル
            public CancellationTokenSource threadCTS;   // 入出力スレッドキャンセル処理用変数
            public UInt32 uiOpeCnt;                     // スレッド動作回数
        }
        //IO_TRDINFO ThreadInfo_Gather = null;                   //収集用スレッドテーブル
        IO_TRDINFO ThreadOutInfo_Dlg = null;            //診断情報用スレッドテーブル
        IO_TRDINFO ThreadSMARTInfo_Dlg = null;              //S.M.A.R.T.情報収集用スレッドテーブル

        /// <summary>
        /// 診断情報テーブル
        /// </summary>
        public class TB_DIAG
        {
            public string TagName = "";
            public IntPtr Entry = IntPtr.Zero;
            public TagPropTHCls CheckValue = null;
            public MntTagCls StatusTag = null;
            public SByte Status = -1;
            public bool IsEnabled = false;
            public bool UsePCounter = false;
            public bool IsErrorOutput = false; //エラーを一度だけ出力するフラグ
            public string PCounterName_All = ""; //カウンタ名
            public uint Adjust_value = 1;
            public TB_DIAG(string _TagName, TagPropTHCls _CheckValue, MntTagCls _StatusTag, bool _UsePCounter, string CounterName_All, uint _Adjustvalue = 1)
            {
                TagName = _TagName;
                CheckValue = _CheckValue;
                StatusTag = _StatusTag;
                UsePCounter = _UsePCounter;
                PCounterName_All = CounterName_All;
                Adjust_value = _Adjustvalue;
            }
            public TB_DIAG(string _TagName, TagPropTHCls _CheckValue, MntTagCls _StatusTag)
            {
                TagName = _TagName;
                CheckValue = _CheckValue;
                StatusTag = _StatusTag;
            }
        }
        Dictionary<string, TB_DIAG> DiagDicTb = new Dictionary<string, TB_DIAG>();

        ////SMART情報のステータス値
        //public enum SMART_STATUS
        //{
        //    GOOD = 0,
        //    GOOD_GREEN = 1,
        //    CAUTION = 2,
        //    BAD = 3,
        //    UNKNOWN = 4,
        //};
        //タグ種別 出力方法が異なるため分ける
        public enum TAG_TBKEY
        {
            DIAG = 0,
            SMART_NAME = 1,
            SMART_CURRENT = 2,
            SMART_STATUS = 3,
            PCOUNTER_UNIQ = 4,
        };
        //タグのエントリーとタグ種別を保持
        public class DiagTag
        {
            public IntPtr Entry = IntPtr.Zero;
            public TAG_TBKEY tbKey = 0;
            public object key1;
            public object key2;

            public DiagTag(IntPtr _Entry, TAG_TBKEY _tbKey)
            {
                Entry = _Entry;
                tbKey = _tbKey;
            }
            public DiagTag(IntPtr _Entry, TAG_TBKEY _tbKey, object _key1, object _key2)
            {
                Entry = _Entry;
                tbKey = _tbKey;
                key1 = _key1;
                key2 = _key2;
            }
        }
        edge_API.DiskExecInfo diskExecInfo = new edge_API.DiskExecInfo();
        
        Dictionary<string, DiagTag> TagOutTb = new Dictionary<string, DiagTag>(); //string = タグ名 , DiagTag




        /// <summary>
        /// コンストラクタ(private)
        /// </summary>
        private EdgeService() { }

        /// <summary>
        /// インスタンス取得関数
        /// --------------------------------------------------------
        /// 使用する際は下記のようにインスタンスを取得してください。
        /// EdgeService service = EdgeService.GetInstance();
        /// </summary>
        /// <returns></returns>
        public static EdgeService GetInstance()
        {
            return service;
        }
        /// <summary>
        /// サービス起動時処理
        /// </summary>
        /// <returns></returns>
        public int Startup(MainWindow Main)
        {
            try
            {
                SetInitStatus(CDefine.INIT_STS_INIT);           // 初期化中に更新

                _Main = Main;

                // RT-edgeの準備
                EGAPI = new edge_API();

                // RT-edgeユーザハンドラ登録
                edge_API.EDGECONFIG config = EGAPI.EDGE_CONFIG_DEFAULT;
                config.mInstMessFunc = MsgHandler;

                // RT-edgeフレームワーク初期化
                var ret = EGAPI.EgInit(config);
                if (ret != 0)
                    PrintLog("ERR", "Failed to initialize. Please check the environment.");

                // サービス名の設定
                if (EGAPI.EGFW.ServiceRealName != null)
                    ServiceRealName = EGAPI.EGFW.ServiceRealName;

                diag = new Diag();            // diagクラス生成

                CreateMntTag();               //内部タグ構造体の作成

                // ライブインジケータタグの周期機能 / Service Live counter timer
                ServiceIndLiveTimer = new ServiceLiveIndicatorTimer(this, 1000);

                SetErrorID(CDefine.SUCCESS);  //エラーコード設定

                // インジケータタグの準備
                CreateIndicater();

                // XMLプロパティタグ設定の取り出し
                GetProperty();

                // Diagテーブルの作成
                CreateDiagTable();

                // 出力スレッド(診断情報出力スレッド)生成
                ThreadOutInfo_Dlg = CreateOutThreadInfo();                       // 出力スレッド情報生成
                if ((UInt64)Cycle.Value > 0)      // サイクル周期の指定がある場合のみスレッド作成
                    ThreadDlgOutService(ThreadOutInfo_Dlg.threadCTS.Token, ThreadOutInfo_Dlg);  // 出力スレッド処理作成

                // S.M.A.R.T.取得スレッド生成
                ThreadSMARTInfo_Dlg = CreateSMARTThreadInfo();
                if ((UInt64)Cycle_SMART.Value > 0)      // サイクル周期の指定がある場合のみスレッド作成
                    ThreadDlgSMARTService(ThreadSMARTInfo_Dlg.threadCTS.Token, ThreadSMARTInfo_Dlg);  // S.M.A.R.T.取得スレッド処理作成

                // ライブインジケータタグの周期更新スタート / Service Live counter timer
                ServiceIndLiveTimer.RefleshStart();
                if (CheckInitRun())                                 // 初期化中(RUN受信済み)の場合
                {
                    // 初期化途中に EM_SERVICE_RUN 受信していた場合は、ここに入る
                    SetInitStatus(CDefine.INIT_STS_INITEND);        // 初期化完了に更新
                    StartService();                                 // サービス開始処理実行
                }
                SetInitStatus(CDefine.INIT_STS_INITEND);        // 初期化完了に更新
                return 0;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }


        /// <summary>
        /// 自己のサービス名を返す
        /// </summary>
        /// <returns></returns>
        public String GetServiceName()
        {
            if (ServiceRealName.Length > 0) //XMLで指定したサービス名 EgInit後に設定 設定がない場合はデフォルト値を返す
                return ServiceRealName;
            return DefServiceName; //"EgBM"
        }
        /// <summary>
        /// サービス+タグ取得
        /// </summary>
        /// <param name="TagTailName"></param>
        /// <returns></returns>
        public String GetServiceTagName(string TagTailName)
        {
            return String.Format("SERVICE.{0}.{1}", GetServiceName(), TagTailName);
        }
        /// <summary>
        /// システムタグ名取得
        /// </summary>
        /// <param name="TagTailName"></param>
        /// <returns></returns>
        public String GetSystemTagName(string TagTailName)
        {
            return String.Format("EDGESYSTEM.{0}", TagTailName);
        }
        /// <summary>
        /// 内部で保持するタグクラスの作成
        /// </summary>
        public void CreateMntTag()
        {
            //変数の初期化
            diskExecInfo.diskInfos = new Dictionary<int, edge_API.DiskInfo>();

            //タグの定義
            ServiceIndRun = new MntTagCls(GetServiceTagName("Run"), EGDEFINE.egTagDataType.Boolean);　     // インジケータタグ".Run" : bool / Service Run indicator
            ServiceIndStatus = new MntTagCls(GetServiceTagName("Status"), EGDEFINE.egTagDataType.Byte);　  // インジケータタグ".Status" : byte / Service Status indicator
            ServiceIndError = new MntTagCls(GetServiceTagName("Error"), EGDEFINE.egTagDataType.Boolean);　   // インジケータタグ".Error" : bool / Service Error indicator
            ServiceIndLive = new MntTagCls(GetServiceTagName("Live"), EGDEFINE.egTagDataType.UInt32);　    // インジケータタグ".Live" : UInt32 / Service Live counter indicator

            ///// <summary>
            ///// システムタグ
            ///// </summary>
            //CpuUse = new MntTagCls(GetSystemTagName("CpuUse"), EGDEFINE.egTagDataType.Float);             // システムタグ"EDGESYSTEM.CpuUse" : float / CPU使用率
            //MemUse = new MntTagCls(GetSystemTagName("MemUse"), EGDEFINE.egTagDataType.Float);             // システムタグ"EDGESYSTEM.MemUse" : float / メモリ使用量
            //AccErr = new MntTagCls(GetSystemTagName("AccErr"), EGDEFINE.egTagDataType.UInt32);             // システムタグ"EDGESYSTEM.AccErr" : UInt32 / メモリ使用量

            /// <summary>
            /// システムタグ EDGESYSTEM.XXXX.Status 正常・異常判定値
            /// </summary>
            CpuUse_Status = new MntTagCls(GetSystemTagName("CpuUse.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.CpuUse.Status
            MemUse_Status = new MntTagCls(GetSystemTagName("MemUse.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.MemUse.Status
            OperatingTime_Status = new MntTagCls(GetSystemTagName("OperatingTime.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.OperatingTime.Status
            AccErr_Status = new MntTagCls(GetSystemTagName("AccErr.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.AccErr.Status

            DiskSpace_A_Status = new MntTagCls(GetSystemTagName("DiskSpace_A.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_A_Status
            DiskSpace_B_Status = new MntTagCls(GetSystemTagName("DiskSpace_B.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_B_Status
            DiskSpace_C_Status = new MntTagCls(GetSystemTagName("DiskSpace_C.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_C_Status
            DiskSpace_D_Status = new MntTagCls(GetSystemTagName("DiskSpace_D.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_D_Status
            DiskSpace_E_Status = new MntTagCls(GetSystemTagName("DiskSpace_E.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_E_Status
            DiskSpace_F_Status = new MntTagCls(GetSystemTagName("DiskSpace_F.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_F_Status
            DiskSpace_G_Status = new MntTagCls(GetSystemTagName("DiskSpace_G.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_G_Status
            DiskSpace_H_Status = new MntTagCls(GetSystemTagName("DiskSpace_H.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_H_Status
            DiskSpace_I_Status = new MntTagCls(GetSystemTagName("DiskSpace_I.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_I_Status
            DiskSpace_J_Status = new MntTagCls(GetSystemTagName("DiskSpace_J.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_J_Status
            DiskSpace_K_Status = new MntTagCls(GetSystemTagName("DiskSpace_K.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_K_Status
            DiskSpace_L_Status = new MntTagCls(GetSystemTagName("DiskSpace_L.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_L_Status
            DiskSpace_M_Status = new MntTagCls(GetSystemTagName("DiskSpace_M.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_M_Status
            DiskSpace_N_Status = new MntTagCls(GetSystemTagName("DiskSpace_N.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_N_Status
            DiskSpace_O_Status = new MntTagCls(GetSystemTagName("DiskSpace_O.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_O_Status
            DiskSpace_P_Status = new MntTagCls(GetSystemTagName("DiskSpace_P.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_P_Status
            DiskSpace_Q_Status = new MntTagCls(GetSystemTagName("DiskSpace_Q.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_Q_Status
            DiskSpace_R_Status = new MntTagCls(GetSystemTagName("DiskSpace_R.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_R_Status
            DiskSpace_S_Status = new MntTagCls(GetSystemTagName("DiskSpace_S.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_S_Status
            DiskSpace_T_Status = new MntTagCls(GetSystemTagName("DiskSpace_T.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_T_Status
            DiskSpace_U_Status = new MntTagCls(GetSystemTagName("DiskSpace_U.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_U_Status
            DiskSpace_V_Status = new MntTagCls(GetSystemTagName("DiskSpace_V.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_V_Status
            DiskSpace_W_Status = new MntTagCls(GetSystemTagName("DiskSpace_W.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_W_Status
            DiskSpace_X_Status = new MntTagCls(GetSystemTagName("DiskSpace_X.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_X_Status
            DiskSpace_Y_Status = new MntTagCls(GetSystemTagName("DiskSpace_Y.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_Y_Status
            DiskSpace_Z_Status = new MntTagCls(GetSystemTagName("DiskSpace_Z.Status"), EGDEFINE.egTagDataType.SByte);//システムタグ EDGESYSTEM.DiskSpace_Z_Status

            /// <summary>
            /// プロパティタグ
            /// </summary>
            Cycle = new MntTagCls(GetServiceTagName("Cycle"), EGDEFINE.egTagDataType.UInt64);               // プロパティタグ".Cycle" : UInt64 診断機能用 診断周期(秒)
            AutoRun = new MntTagCls(GetServiceTagName("AutoRun"), EGDEFINE.egTagDataType.Boolean);             // プロパティタグ".AutoRun" : bool 動作自動スタートの有効有無
            Error = new MntTagCls(GetServiceTagName("Error"), EGDEFINE.egTagDataType.Boolean);               // インジケータタグ".Error" : bool エラー状態の有効有無
            ErrorID = new MntTagCls(GetServiceTagName("ErrorID"), EGDEFINE.egTagDataType.UInt32);             // インジケータタグ".ErrorID" UInt32 実行失敗時のエラーコード
            ExecuteCnt = new MntTagCls(GetServiceTagName("ExecuteCnt"), EGDEFINE.egTagDataType.UInt32);          // 統計情報タグ".ExecuteCnt" UInt32 実行数
            ThreadCnt = new MntTagCls(GetServiceTagName("ThreadCnt"), EGDEFINE.egTagDataType.UInt32);          // 統計情報タグ".ExecuteCnt" UInt32 実行数

            Diag_CpuUseTH = new TagPropTHCls(GetServiceTagName("CpuUseTH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);    // プロパティタグ".CpuUseTH" : Float CPU使用率 閾値 (%)
            Diag_MemUseTH = new TagPropTHCls(GetServiceTagName("MemUseTH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE); ;    // プロパティタグ".MemUseTH" : Float メモリ使用量 閾値 (MB)
            Diag_OpeTimeTH = new TagPropTHCls(GetServiceTagName("OpeTimeTH"), EGDEFINE.egTagDataType.UInt64, DIAG_THMODE_NOTDEFINE); ;   // プロパティタグ".OpeTimeTH" : UInt64 システム稼働時間 閾値 (秒)
            Diag_AccErrTH = new TagPropTHCls(GetServiceTagName("AccErrTH"), EGDEFINE.egTagDataType.UInt32, DIAG_THMODE_NOTDEFINE); ;    // プロパティタグ".AccErrTH" : UInt32 周辺機器エラー数 閾値 (個)

            Cycle_SMART = new MntTagCls(GetServiceTagName("Cycle_SMART"), EGDEFINE.egTagDataType.UInt64);               // プロパティタグ".Cycle_SMART" : UInt64 診断機能用 診断周期(分)(S.M.A.R.T.用)

            /// <summary>
            /// プロパティタグ SERVICE.EgMDiag.DiskSpace_{Drive}_TH ディスク空き容量
            /// </summary>
            Diag_DiskSpace_A_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_A_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_A_TH
            Diag_DiskSpace_B_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_B_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_B_TH
            Diag_DiskSpace_C_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_C_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_C_TH
            Diag_DiskSpace_D_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_D_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_D_TH
            Diag_DiskSpace_E_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_E_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_E_TH
            Diag_DiskSpace_F_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_F_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_F_TH
            Diag_DiskSpace_G_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_G_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_G_TH
            Diag_DiskSpace_H_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_H_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_H_TH
            Diag_DiskSpace_I_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_I_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_I_TH
            Diag_DiskSpace_J_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_J_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_J_TH
            Diag_DiskSpace_K_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_K_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_K_TH
            Diag_DiskSpace_L_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_L_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_L_TH
            Diag_DiskSpace_M_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_M_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_M_TH
            Diag_DiskSpace_N_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_N_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_N_TH
            Diag_DiskSpace_O_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_O_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_O_TH
            Diag_DiskSpace_P_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_P_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_P_TH
            Diag_DiskSpace_Q_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_Q_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_Q_TH
            Diag_DiskSpace_R_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_R_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_R_TH
            Diag_DiskSpace_S_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_S_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_S_TH
            Diag_DiskSpace_T_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_T_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_T_TH
            Diag_DiskSpace_U_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_U_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_U_TH
            Diag_DiskSpace_V_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_V_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_V_TH
            Diag_DiskSpace_W_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_W_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_W_TH
            Diag_DiskSpace_X_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_X_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_X_TH
            Diag_DiskSpace_Y_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_Y_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_Y_TH
            Diag_DiskSpace_Z_TH = new TagPropTHCls(GetServiceTagName("DiskSpace_Z_TH"), EGDEFINE.egTagDataType.Float, DIAG_THMODE_NOTDEFINE);//プロパティタグ SERVICE.EgMDiag.DiskSpace_Z_TH
        }
        /// <summary>
        /// 初期化状態更新
        /// </summary>
        /// <param name="sts"></param>
        public void SetInitStatus(byte sts)
        {
            InitState = sts;
        }
        /// <summary>
        /// 初期化完了済みか？(true:済み,false:未)
        /// </summary>
        /// <returns></returns>
        public bool CheckInitEnd()
        {
            return (InitState == CDefine.INIT_STS_INITEND);
        }
        /// <summary>
        /// 初期化中(RUN受信済み)か？(true:初期化中(RUN受信済み),false:未)
        /// </summary>
        /// <returns></returns>
        public bool CheckInitRun()
        {
            return (InitState == CDefine.INIT_STS_INITRUN);
        }
        ///*****************************************************************************
        /// <summary>
        /// サービス開始 ※入出力可能になります
        /// 必要が無ければ変更不要
        /// </summary>
        /// <returns>正常時０</returns>
        ///*****************************************************************************
        public int StartService()
        {
            try
            {
                ServiceIndRun.Value = true;                           // サービス実行状態に更新
                return 0;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// ErrorIDの設定
        /// </summary>
        /// <param name="_ErrorID"></param>
        public void SetErrorID(uint _ErrorID)
        {
            ErrorID.Value = _ErrorID;
            if (_ErrorID == 0)  //0の時は成功 Errorフラグも変更
                ServiceIndError.Value = false;
            else
                ServiceIndError.Value = true;
        }
        /// <summary>
        /// インジケータタグの生成
        /// </summary>
        /// <returns></returns>
        public int CreateIndicater()
        {
            int ret = 0;

            // インジケータタグを生成します
            ret |= EGAPI.EgTagCreate(String.Format("SERVICE.{0}.Status", GetServiceName()), (ushort)EGDEFINE.egTagDataType.Byte, 1, "", "service status");
            ret |= EGAPI.EgTagCreate(String.Format("SERVICE.{0}.Error", GetServiceName()), (ushort)EGDEFINE.egTagDataType.Byte, 1, "", "service status");
            ret |= EGAPI.EgTagCreate(String.Format("SERVICE.{0}.ErrorID", GetServiceName()), (ushort)EGDEFINE.egTagDataType.UInt32, 4, "", "service etc last Error ID");
            ret |= EGAPI.EgTagCreate(String.Format("SERVICE.{0}.Run", GetServiceName()), (ushort)EGDEFINE.egTagDataType.Boolean, 1, "", "service runnning");

            //各値初期化
            ServiceIndError.Value = false;
            ErrorID.Value = (UInt32)0;
            ExecuteCnt.Value = (UInt32)0;
            
            return ret;
        }
        /// <summary>
        /// サービスプロパティのXML解釈処理
        /// </summary>
        /// <returns></returns>
        public bool GetProperty()
        {
            bool ret = true;
            ret |= GetTagValueTagCls(ref Cycle, (UInt64)0);// Diag.Cycleオプションの利用（デフォルト 0 周期で収集しない）

            ret |= GetTagValueTagCls(ref Cycle_SMART, (UInt64)0); // Diag.Cycle_SMARTオプションの利用（デフォルト 0 周期で収集しない）
            // 閾値取得
            ret |= GetTagValueTagPropCls(ref Diag_CpuUseTH, (float)90);// CpuUseTHオプションの利用（デフォルト 90%）
            ret |= GetTagValueTagPropCls(ref Diag_MemUseTH, (float)3072);// Diag.MemUseTHオプションの利用（デフォルト 3072 MB）

            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_A_TH, (float)0);// Diag.DiskSpace_A_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_B_TH, (float)0);// Diag.DiskSpace_B_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_C_TH, (float)0);// Diag.DiskSpace_C_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_D_TH, (float)0);// Diag.DiskSpace_D_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_E_TH, (float)0);// Diag.DiskSpace_E_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_F_TH, (float)0);// Diag.DiskSpace_F_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_G_TH, (float)0);// Diag.DiskSpace_G_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_H_TH, (float)0);// Diag.DiskSpace_H_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_I_TH, (float)0);// Diag.DiskSpace_I_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_J_TH, (float)0);// Diag.DiskSpace_J_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_K_TH, (float)0);// Diag.DiskSpace_K_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_L_TH, (float)0);// Diag.DiskSpace_L_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_M_TH, (float)0);// Diag.DiskSpace_M_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_N_TH, (float)0);// Diag.DiskSpace_N_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_O_TH, (float)0);// Diag.DiskSpace_O_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_P_TH, (float)0);// Diag.DiskSpace_P_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_Q_TH, (float)0);// Diag.DiskSpace_Q_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_R_TH, (float)0);// Diag.DiskSpace_R_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_S_TH, (float)0);// Diag.DiskSpace_S_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_T_TH, (float)0);// Diag.DiskSpace_T_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_U_TH, (float)0);// Diag.DiskSpace_U_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_V_TH, (float)0);// Diag.DiskSpace_V_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_W_TH, (float)0);// Diag.DiskSpace_W_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_X_TH, (float)0);// Diag.DiskSpace_X_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_Y_TH, (float)0);// Diag.DiskSpace_Y_THオプションの利用（デフォルト 0）
            ret |= GetTagValueTagPropCls(ref Diag_DiskSpace_Z_TH, (float)0);// Diag.DiskSpace_Z_THオプションの利用（デフォルト 0）

            ret |= GetTagValueTagPropCls(ref Diag_OpeTimeTH, (UInt64)2592000);// Diag.OpeTimeTHオプションの利用（デフォルト 2592000 秒）
            ret |= GetTagValueTagPropCls(ref Diag_AccErrTH, (UInt32)1);// Diag.AccErrTHオプションの利用（デフォルト 1 個）

            ret |= GetTagValueTagCls(ref AutoRun, (bool)false);//動作自動スタートの有効有無
            
            return ret;
        }
        /// <summary>
        /// 診断用テーブル
        /// ※本当はxmlから作りたい　何とかしたい・・・今は強引で
        /// </summary>
        void CreateDiagTable()
        {
            try
            {
                DiagDicTb.Add("EDGESYSTEM.CpuUse", new TB_DIAG("EDGESYSTEM.CpuUse", Diag_CpuUseTH, CpuUse_Status, true, "Processor Information(_Total)\\% Processor Utility"));
                DiagDicTb.Add("EDGESYSTEM.MemUse", new TB_DIAG("EDGESYSTEM.MemUse", Diag_MemUseTH, MemUse_Status, true, "Process(_Total)\\Working Set", 1024 * 1024));
                DiagDicTb.Add("EDGESYSTEM.OperatingTime", new TB_DIAG("EDGESYSTEM.OperatingTime", Diag_OpeTimeTH, OperatingTime_Status));
                DiagDicTb.Add("EDGESYSTEM.AccErr", new TB_DIAG("EDGESYSTEM.AccErr", Diag_AccErrTH, AccErr_Status));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_A", new TB_DIAG("EDGESYSTEM.DiskSpace_A", Diag_DiskSpace_A_TH, DiskSpace_A_Status, true, "LogicalDisk(A:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_B", new TB_DIAG("EDGESYSTEM.DiskSpace_B", Diag_DiskSpace_B_TH, DiskSpace_B_Status, true, "LogicalDisk(B:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_C", new TB_DIAG("EDGESYSTEM.DiskSpace_C", Diag_DiskSpace_C_TH, DiskSpace_C_Status, true, "LogicalDisk(C:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_D", new TB_DIAG("EDGESYSTEM.DiskSpace_D", Diag_DiskSpace_D_TH, DiskSpace_D_Status, true, "LogicalDisk(D:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_E", new TB_DIAG("EDGESYSTEM.DiskSpace_E", Diag_DiskSpace_E_TH, DiskSpace_E_Status, true, "LogicalDisk(E:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_F", new TB_DIAG("EDGESYSTEM.DiskSpace_F", Diag_DiskSpace_F_TH, DiskSpace_F_Status, true, "LogicalDisk(F:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_G", new TB_DIAG("EDGESYSTEM.DiskSpace_G", Diag_DiskSpace_G_TH, DiskSpace_G_Status, true, "LogicalDisk(G:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_H", new TB_DIAG("EDGESYSTEM.DiskSpace_H", Diag_DiskSpace_H_TH, DiskSpace_H_Status, true, "LogicalDisk(H:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_I", new TB_DIAG("EDGESYSTEM.DiskSpace_I", Diag_DiskSpace_I_TH, DiskSpace_I_Status, true, "LogicalDisk(I:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_J", new TB_DIAG("EDGESYSTEM.DiskSpace_J", Diag_DiskSpace_J_TH, DiskSpace_J_Status, true, "LogicalDisk(J:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_K", new TB_DIAG("EDGESYSTEM.DiskSpace_K", Diag_DiskSpace_K_TH, DiskSpace_K_Status, true, "LogicalDisk(K:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_L", new TB_DIAG("EDGESYSTEM.DiskSpace_L", Diag_DiskSpace_L_TH, DiskSpace_L_Status, true, "LogicalDisk(L:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_M", new TB_DIAG("EDGESYSTEM.DiskSpace_M", Diag_DiskSpace_M_TH, DiskSpace_M_Status, true, "LogicalDisk(M:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_N", new TB_DIAG("EDGESYSTEM.DiskSpace_N", Diag_DiskSpace_N_TH, DiskSpace_N_Status, true, "LogicalDisk(N:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_O", new TB_DIAG("EDGESYSTEM.DiskSpace_O", Diag_DiskSpace_O_TH, DiskSpace_O_Status, true, "LogicalDisk(O:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_P", new TB_DIAG("EDGESYSTEM.DiskSpace_P", Diag_DiskSpace_P_TH, DiskSpace_P_Status, true, "LogicalDisk(P:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_Q", new TB_DIAG("EDGESYSTEM.DiskSpace_Q", Diag_DiskSpace_Q_TH, DiskSpace_Q_Status, true, "LogicalDisk(Q:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_R", new TB_DIAG("EDGESYSTEM.DiskSpace_R", Diag_DiskSpace_R_TH, DiskSpace_R_Status, true, "LogicalDisk(R:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_S", new TB_DIAG("EDGESYSTEM.DiskSpace_S", Diag_DiskSpace_S_TH, DiskSpace_S_Status, true, "LogicalDisk(S:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_T", new TB_DIAG("EDGESYSTEM.DiskSpace_T", Diag_DiskSpace_T_TH, DiskSpace_T_Status, true, "LogicalDisk(T:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_U", new TB_DIAG("EDGESYSTEM.DiskSpace_U", Diag_DiskSpace_U_TH, DiskSpace_U_Status, true, "LogicalDisk(U:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_V", new TB_DIAG("EDGESYSTEM.DiskSpace_V", Diag_DiskSpace_V_TH, DiskSpace_V_Status, true, "LogicalDisk(V:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_W", new TB_DIAG("EDGESYSTEM.DiskSpace_W", Diag_DiskSpace_W_TH, DiskSpace_W_Status, true, "LogicalDisk(W:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_X", new TB_DIAG("EDGESYSTEM.DiskSpace_X", Diag_DiskSpace_X_TH, DiskSpace_X_Status, true, "LogicalDisk(X:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_Y", new TB_DIAG("EDGESYSTEM.DiskSpace_Y", Diag_DiskSpace_Y_TH, DiskSpace_Y_Status, true, "LogicalDisk(Y:)\\% Free Space"));
                DiagDicTb.Add("EDGESYSTEM.DiskSpace_Z", new TB_DIAG("EDGESYSTEM.DiskSpace_Z", Diag_DiskSpace_Z_TH, DiskSpace_Z_Status, true, "LogicalDisk(Z:)\\% Free Space"));
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        ///*****************************************************************************
        /// <summary>
        /// 出力スレッド情報生成
        /// </summary>
        /// <param name="usID"></param>
        /// <returns></returns>
        ///*****************************************************************************
        public IO_TRDINFO CreateOutThreadInfo()
        {
            IO_TRDINFO trdInfo = new IO_TRDINFO();
            trdInfo.hTrigSem = new SemaphoreSlim(1, 1);         // 入力サービススレッド用トリガセマフォ
            trdInfo.threadCTS = new CancellationTokenSource();  // スレッド終了用token指定
            return trdInfo;
        }
        ///*****************************************************************************
        /// <summary>
        /// 診断情報出力スレッド
        /// </summary>
        /// <param name="cancelToken"></param>
        /// <param name="trdInfo"></param>
        ///*****************************************************************************
        public async void ThreadDlgOutService(CancellationToken cancelToken, IO_TRDINFO trdInfo)
        {
            try
            {
                if (EGAPI.EGFW.tagRef_OUT_List.Count < 1)               //TagRefが登録されていない場合は処理しない
                {
                    PrintLog("ERR", "Stop OUT Service. Because TagRef(TagRefs_OUT) is NULL.");
                    SetErrorID(CDefine.ERR_DLG_NOTTARGETTAG);       // エラーインジケータをtrueに更新
                    return;
                }

                await Task.Run(async () =>
                {
                    while (true)
                    {
                        await trdInfo.hTrigSem.WaitAsync();
                        // キャンセルトークンが来たら終了
                        if (cancelToken.IsCancellationRequested)
                            break;

                        //サイクル指定がある場合のみ処理を行う
                        if ((UInt64)Cycle.Value > 0)
                        {
                            if (CheckInitEnd() == true && (bool)ServiceIndRun.Value == true) //Runがtrueの場合のみ実行
                            {
                                //TagRef_OUTを回して処理
                                ExecuteDlgOutTagRefOUT();
                                ThreadCnt.Value = ++(trdInfo.uiOpeCnt);          //スレッド処理回数をTagへ書き込む
                            }
                            Thread.Sleep(TimeSpan.FromSeconds((UInt64)Cycle.Value));//秒でSleep
                            trdInfo.hTrigSem.Release();//終わったらリリースする事で実現
                        }
                    }
                });
            }
            catch (Exception ex)
            {
                PrintLog("ERR", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// 閾値チェック＋タグの書込み
        /// </summary>
        /// <param name="Diag"></param>
        /// <param name="value"></param>
        void CheckWriteTagTH(ref TB_DIAG Diag, object value)
        {
            //閾値が定義されている場合のみ実施
            if (Diag.CheckValue != null && Diag.CheckValue.IsTargetIsBig != DIAG_THMODE_NOTDEFINE)
            {
                //値の比較
                SByte sbState = Diag.CheckValue.CheckTH(value);          //閾値チェック
                Diag.StatusTag.Value = sbState;    //タグの書込み
            }
            else
            {
                if (Diag.IsErrorOutput == false)
                {
                    //設定がおかしい 1回だけ出力
                    PrintLog("WRN", "診断情報 tag[" + Diag.TagName + "] の設定が無い為チェックを行いませんでした。");
                }
            }
            Diag.IsErrorOutput = true;//2度目以降はエラーを出させない
        }
        //
        //
        //
        bool fnSetPCounterTag(EgHgTagRef tagRef, ref Dictionary<string, DiagTag> _TagOutTb)
        {
            try
            {
                string inputAddress = GetTagAddress(tagRef.Name);
                if (inputAddress.Length == 0) //対象ではないので追加しない
                {
                    PrintLog("ERR", String.Format("'{0}': Address値が存在しない為診断対象外とします。", tagRef.Name));
                    return false;
                }
                int ret = EGAPI.EgMDiagExecute_PfmCntAdd(inputAddress);
                if(ret == 0) //成功時
                { 
                    //閾値タグの登録
                    TagPropTHCls tagTHCls = new TagPropTHCls(tagRef.Name + "_TH");

                    //タグタイプ設定
                    if (SetTagTypeTagPropCls(ref tagTHCls) == 0)
                    {
                        //存在しない場合はnullとする 実行時にエラー
                        if (GetTagValueTagPropCls(ref tagTHCls, 0) == false)
                            tagTHCls = null;
                    }
                    else//存在しない場合はnullとする 実行時にエラー
                        tagTHCls = null;


                    //ステータスタグの登録　存在しない場合は実行時に何もしない
                    MntTagCls tagMntCls = new MntTagCls(tagRef.Name + ".Status", EGDEFINE.egTagDataType.SByte);//ステータスタグ TAGNAME.Status

                    //診断用に追加
                    DiagDicTb.Add(tagRef.Name,new TB_DIAG(tagRef.Name, tagTHCls, tagMntCls, true, inputAddress));

                    // 種別分け結果を追加
                    _TagOutTb.Add(tagRef.Name, new DiagTag(tagRef.Entry, TAG_TBKEY.DIAG));

                }
                else
                {
                    PrintLog("ERR", String.Format("'{0}': Address値の入力形式が不正の為、診断対象外とします。", tagRef.Name));
                    return false;
                }
                return true;
            }
            catch
            {
                PrintLog("ERR", String.Format("'{0}': Address値の入力形式が不正の為、診断対象外とします。", tagRef.Name));
                return false;
            }
        }


        /// <summary>
        /// タグの種類分け
        /// </summary>
        /// <param name="TagList"></param>
        void fnSetTagKind(List<EgHgTagRef> TagList)
        {
            for (int i = 0; i < TagList.Count; i++)
            {
                EgHgTagRef tagRef = TagList[i];
                string modifiedName = tagRef.Name.Replace("EDGESYSTEM.SMART.", ""); //EDが16進数の文字列として認識されてしまうため

                var matches = Regex.Matches(modifiedName, @"[0-9A-F]{2}");
                //カレント値
                if (Regex.IsMatch(tagRef.Name, @"^EDGESYSTEM\.SMART\.[0-9A-F]{2}\.[0-9A-F]{2}$"))
                {
                    if (matches.Count >= 2)
                        TagOutTb.Add(tagRef.Name, new DiagTag(tagRef.Entry, TAG_TBKEY.SMART_CURRENT, Convert.ToByte(matches[0].Value, 16), Convert.ToByte(matches[1].Value, 16)));
                }
                //ステータス値
                else if (Regex.IsMatch(tagRef.Name, @"^EDGESYSTEM\.SMART\.[0-9A-F]{2}\.[0-9A-F]{2}\.Status$"))
                {
                    if (matches.Count >= 2)
                        TagOutTb.Add(tagRef.Name, new DiagTag(tagRef.Entry, TAG_TBKEY.SMART_STATUS, Convert.ToByte(matches[0].Value, 16), Convert.ToByte(matches[1].Value, 16)));

                }
                //名前
                else if (Regex.IsMatch(tagRef.Name, @"^EDGESYSTEM\.SMART\.[0-9A-F]{2}\.Model$"))
                {
                    if (matches.Count >= 1)
                        TagOutTb.Add(tagRef.Name, new DiagTag(tagRef.Entry, TAG_TBKEY.SMART_NAME, Convert.ToByte(matches[0].Value, 16), 0));
                }
                else
                {
                    //
                    // 最後の文字が .Counter の場合 任意パフォーマンスカウンタタグとして扱う
                    //
                    if (tagRef.Name.EndsWith(".Counter"))
                        fnSetPCounterTag(tagRef, ref TagOutTb);//エラーは内部で出力
                    else //その他 既存のタグ(DIAG)とみなす
                        TagOutTb.Add(tagRef.Name, new DiagTag(tagRef.Entry, TAG_TBKEY.DIAG));
                }
            }
        }
        /// <summary>
        /// 診断機能実行
        /// </summary>
        void ExecuteDlgOutTagRefOUT()
        {
            try
            {
                if ((bool)ServiceIndRun.Value == false) //Runがfalseの場合は実行しない
                    return;

                List<EgHgTagRef> TagList = EGAPI.EGFW.tagRef_OUT_List;
                //
                // 最初だけ、S.M.A.R.T.情報出力用タグ・ステータスタグを内部で作成
                //
                if (bFirst_SetTagKind_Flag == false)
                {
                    fnSetTagKind(TagList);
                    bFirst_SetTagKind_Flag = true;
                }

                foreach (KeyValuePair<string, DiagTag> tagout in TagOutTb)
                {
                    //S.M.A.R.T情報以外か
                    if (tagout.Value.tbKey == TAG_TBKEY.DIAG)
                    {
                        if (DiagDicTb.ContainsKey(tagout.Key) == true)
                        {
                            TB_DIAG Diag = DiagDicTb[tagout.Key];
                            if (Diag != null)//存在する
                            {
                                if (Diag.UsePCounter == true)//パフォーマンスカウンタを使用する
                                {
                                    float flValue = 0;
                                    if (true == diag.GetPerformanceCounterOne(Diag.PCounterName_All, ref flValue)) //値を取得
                                    {
                                        if (flValue == 0) //もう一度呼ぶ
                                        {
                                            diag.GetPerformanceCounterOne(Diag.PCounterName_All, ref flValue);
                                        }
                                        if (flValue != 0 && Diag.Adjust_value != 1)
                                            flValue = flValue / Diag.Adjust_value;
                                        //Tagに反映
                                        EGAPI.EgTagWriteByEntry(tagout.Value.Entry, flValue);

                                        // チェック 閾値が定義されている場合のみ実施
                                        CheckWriteTagTH(ref Diag, flValue);
                                    }
                                }
                                else//それ以外は別処理
                                {
                                    //現在 周辺機器エラーと時間のみ 文字列で比較
                                    //周辺機器エラー
                                    if (String.Compare(Diag.TagName, "EDGESYSTEM.AccErr", true) == 0)
                                    {
                                        int errCount = 0;
                                        if (true == diag.GetAccError(Diag.IsErrorOutput, ref errCount)) //値を取得
                                        {
                                            //Tagに反映
                                            EGAPI.EgTagWriteByEntry(tagout.Value.Entry, errCount);
                                            if (errCount > 0)
                                            {
                                                // チェック 閾値が定義されている場合のみ実施
                                                CheckWriteTagTH(ref Diag, (UInt32)errCount);
                                            }
                                        }
                                    }
                                    //時間
                                    else if (String.Compare(Diag.TagName, "EDGESYSTEM.OperatingTime", true) == 0)
                                    {
                                        object opeObj = (UInt64)0;
                                        bool ret = GetTagValue("EDGESYSTEM.OperatingTime", EGDEFINE.egTagDataType.UInt64, 0, ref opeObj);//値を取得
                                        if (ret)
                                        {
                                            // チェック 閾値が定義されている場合のみ実施
                                            CheckWriteTagTH(ref Diag, opeObj);
                                        }
                                    }
                                }
                            }
                        }
                    }
                    else
                    {
                        //対象のタグが登録されている場合
                        if (tagout.Value.Entry != null)
                        {
                            //S.M.A.R.T のディスク名の場合
                            if (tagout.Value.tbKey == TAG_TBKEY.SMART_NAME)
                            {
                                string ModelName = "N/A";
                                EGAPI.EgMDiagGetSMARTDiskName(diskExecInfo, (byte)tagout.Value.key1, ref ModelName);
                                //Tagに反映
                                EGAPI.EgTagWriteString(tagout.Key, ModelName);
                            }
                            //S.M.A.R.T のカレント値の場合
                            else if (tagout.Value.tbKey == TAG_TBKEY.SMART_CURRENT)
                            {
                                byte Value = 0;
                                byte Status = (byte)EGDEFINE.SMART_STATUS.UNKNOWN;
                                EGAPI.EgMDiagGetSMARTValue(diskExecInfo, (byte)tagout.Value.key1, (byte)tagout.Value.key2, ref Value, ref Status);
                                //Tagに反映
                                EGAPI.EgTagWriteByEntry(tagout.Value.Entry, Value);
                            }//S.M.A.R.T のステータス値の場合
                            else if (tagout.Value.tbKey == TAG_TBKEY.SMART_STATUS)
                            {
                                //Tagに反映
                                byte Value = 0;
                                byte Status = (byte)EGDEFINE.SMART_STATUS.UNKNOWN;
                                EGAPI.EgMDiagGetSMARTValue(diskExecInfo, (byte)tagout.Value.key1, (byte)tagout.Value.key2, ref Value, ref Status);
                                //Tagに反映
                                EGAPI.EgTagWriteByEntry(tagout.Value.Entry, Status);
                            }
                        }
                    }
                }
                ExecuteCnt.Value = (UInt32)ExecuteCnt.Value + (UInt32)1;    //統計情報カウント
            }
            catch (Exception ex)
            {
                PrintLog("ERR", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// Tagの値取得
        /// </summary>
        /// <param name="TagName"></param>
        /// <param name="Type"></param>
        /// <param name="defvalue"></param>
        /// <param name="target"></param>
        /// <returns></returns>
        public bool GetTagValue(string TagName, EGDEFINE.egTagDataType Type, object defvalue, ref object target)
        {
            try
            {
                target = defvalue;              //デフォルト値設定
                int ret = 0;                    //戻り値
                string strValue = "";           //string型取得時のテンポラリーオブジェクト
                object value = new object();    //string型以外取得時のテンポラリーオブジェクト

                //string型の場合は関数が違うため先に処理
                if (Type == EGDEFINE.egTagDataType.String)
                {
                    bool bRet = fnTagReadString(TagName, ref strValue);
                    if (bRet == false)
                        return false;
                    
                }
                else
                {
                    //それ以外は以下の関数
                    ret = EGAPI.EgTagRead(TagName, ref value);
                    if (ret != 0)
                    {
                        //メッセージは出さない 多くなるので。 PrintLog("DBG", String.Format("'{0}': Not Defined XML.", TagName));
                        return false;
                    }
                }
                switch (Type)
                {
                    case EGDEFINE.egTagDataType.Boolean:
                        target = (ret == 0) ? Convert.ToBoolean(value) : (Boolean)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.SByte:
                        target = (ret == 0) ? Convert.ToSByte(value) : (SByte)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.Byte:
                        target = (ret == 0) ? Convert.ToByte(value) : (Byte)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.Int16:
                        target = (ret == 0) ? Convert.ToInt16(value) : (Int16)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.UInt16:
                        target = (ret == 0) ? Convert.ToUInt16(value) : (UInt16)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.Int32:
                        target = (ret == 0) ? Convert.ToInt32(value) : (Int32)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.UInt32:
                        target = (ret == 0) ? Convert.ToUInt32(value) : (UInt32)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.Int64:
                        target = (ret == 0) ? Convert.ToInt64(value) : (Int64)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.UInt64:
                        target = (ret == 0) ? Convert.ToUInt64(value) : (UInt64)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.Float:
                        target = (ret == 0) ? Convert.ToSingle(value) : (float)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.Double:
                        target = (ret == 0) ? Convert.ToDouble(value) : (double)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.String:
                        target = (ret == 0) ? strValue : "";
                        break;
                    default:
                        break;
                }
                //if (Type == EGDEFINE.egTagDataType.String)  //string型の場合とその他で変数が違うので分けて処理
                //    PrintLog("DBG", String.Format("'{0}': {1}", TagName, strValue));
                //else//それ以外は以下の関数
                //    PrintLog("DBG", String.Format("'{0}': {1}", TagName, value));

                return true;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return false;
            }
        }
        /// <summary>
        /// タイプごとにタグの値を取得して設定 MntTagCls指定
        /// </summary>
        /// <param name="Type"></param>
        /// <param name="TagTailName"></param>
        /// <param name="defvalue"></param>
        public bool GetTagValueTagCls(ref MntTagCls myTag, object defvalue)
        {
            try
            {
                object target = new object();
                EGDEFINE.egTagDataType Type = myTag.Type;   //型取得
                string TagName = myTag.TagName;             //Tag名取得
                bool ret = GetTagValue(myTag.TagName, myTag.Type, defvalue, ref target);//Tag値取得
                if (ret == true)
                    myTag.Value = target;
                return ret;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return false;
            }
        }
        /// <summary>
        /// タグの値を取得
        /// </summary>
        /// <param name="myTag"></param>
        /// <param name="defvalue"></param>
        /// <returns></returns>
        public bool GetTagValueTagPropCls(ref TagPropTHCls myTag, object defvalue)
        {
            try
            {
                object target = new object();
                string TagName = myTag.TagName;                   //Tag名取得
                bool ret = GetTagValue(myTag.TagName, myTag.Type, defvalue, ref target);//Tag値取得
                if (ret == true)//存在する場合Entryとbigフラグを取得
                {
                    myTag.Value = target;                               //設定値の設定
                    IntPtr tagPtr = IntPtr.Zero;
                    myTag.IsTargetIsBig = IsTargetBigFlag(TagName);     //大小比較の設定
                    return true;
                }
                return false;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return false;
            }
        }
        /// <summary>
        /// タグのTypeを設定
        /// </summary>
        /// <param name="myTag"></param>
        /// <returns></returns>
        public int SetTagTypeTagPropCls(ref TagPropTHCls myTag)
        {
            try
            {
                //型取得
                int iret = GetTagType(myTag.TagName, ref myTag.Type);

                return iret;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// Tagのアドレスを取得
        /// </summary>
        /// <param name="TagName"></param>
        /// <returns></returns>
        public string GetTagAddress(string TagName)
        {
            string retValue = "";
            try
            {
                //アドレスの取得
                object retObj = new object();
                int ret = EGAPI.EgTagGetProperty(TagName, "Address", ref retObj);
                if (ret == 0)//取得できたら文字列に変換して返す
                {
                    retValue = retObj.ToString();
                    return retValue;
                }

                return retValue;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return retValue;
            }
        }
        /// <summary>
        /// タグのタイプを取得
        /// </summary>
        /// <param name="TagName"></param>
        /// <param name="ref_type"></param>
        /// <returns></returns>
        public int GetTagType(string TagName, ref EGDEFINE.egTagDataType ref_type)
        {
            ref_type = 0;
            try
            {
                object rtnObj = null;
                //タグのサイズ取得
                int ret = EGAPI.EgTagGetProperty(TagName, "Type", ref rtnObj);
                if (ret != 0)
                {
                    string mess = "Error EgTagGetProperty. Name = " + TagName + "  ErrorId = " + ret.ToString();
                    Debug.WriteLine(mess);
                    return ret;
                }
                ref_type = (EGDEFINE.egTagDataType)((ushort)rtnObj);
                return 0;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// 文字列取得用関数 サイズが分からない為取得後読込
        /// </summary>
        /// <param name="TagName"></param>
        /// <param name="strValue"></param>
        /// <returns></returns>
        private bool GetTagEntry(string TagName, ref IntPtr entry)
        {
            try
            {
                int ret = 0;        //戻り値

                object tagPtr = new object();
                ret = EGAPI.EgTagGetProperty(TagName, "Entry", ref tagPtr);
                if (ret == 0)
                {
                    entry = (IntPtr)tagPtr;
                    return true;
                }
                else
                    PrintLog("DBG", String.Format("'{0}': Not Defined XML.", TagName));

                return false;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return false;
            }
        }
        /// <summary>
        /// 閾値チェック時の大小比較フラグ取得
        /// </summary>
        /// <param name="TagName"></param>
        /// <returns></returns>
        private SByte IsTargetBigFlag(string TagName)
        {
            string str = GetTagAddress(TagName);
            if (str.Length > 0)
            {
                if (String.Compare(str, "big", true) == 0)
                    return DIAG_THMODE_BIG;
                else if (String.Compare(str, "small", true) == 0)
                    return DIAG_THMODE_SMALL;
                return DIAG_THMODE_NOTDEFINE;
            }
            return DIAG_THMODE_NOTDEFINE;
        }
        /// <summary>
        /// 文字列取得用関数 サイズが分からない為取得後読込
        /// </summary>
        /// <param name="TagName"></param>
        /// <param name="strValue"></param>
        /// <returns></returns>
        private bool fnTagReadString(string TagName, ref string strValue)
        {
            try
            {
                int ret = 0;        //戻り値
                object strsize = 0;    //サイズ取得用
                ret = EGAPI.EgTagGetProperty(TagName, "Size", ref strsize);
                if (ret == 0)
                {
                    ret = EGAPI.EgTagReadString(TagName, ref strValue, (ushort)strsize);
                    if (ret == 0)
                        return true;
                    else
                        PrintLog("ERR", String.Format("'{0}': TagReadString Error[{1}].", TagName, ret));
                }
                else
                    PrintLog("DBG", String.Format("'{0}': Not Defined XML.", TagName));

                return false;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return false;
            }
        }
        /// <summary>
        /// 診断機能 開始
        /// </summary>
        public void fnDiagOn()
        {
            //
            //実行処理
            //
            ExecuteDlgOutTagRefOUT();
            return;
        }
        /// <summary>
        /// タグトリガー処理
        /// </summary>
        /// <param name="paramArray"></param>
        public void fnTagTriggerOn(byte[] paramArray)
        {
            try
            {
                if (EGAPI.EGFW.tagTriggerList.Count < 1) //タグトリガーの登録が無い場合は処理しない
                    return;

                // バイト配列の先頭2バイトをushort型に変換
                ushort tablecnt = BitConverter.ToUInt16(paramArray, 0);
                if (tablecnt == 0) //テーブル数 0の時は何もしない
                    return;
                if (tablecnt > 100) //100個以上は処理しない
                    return;

                // 構造体の配列に変換
                int structSize = Marshal.SizeOf<EGTAGTRIGINFO>();
                EGTAGTRIGINFO[] structArray = new EGTAGTRIGINFO[tablecnt];
                for (int i = 0; i < tablecnt; i++)
                {
                    byte[] structBytes = new byte[structSize];
                    Array.Copy(paramArray, (i * structSize) + 2, structBytes, 0, structSize); //3バイト目以降を変換
                    structArray[i] = ByteArrayToStructure<EGTAGTRIGINFO>(structBytes);
                    if (EGAPI.EGFW.tagTriggerList.Count > structArray[i].arrNo) //存在する場合のみ実施
                    {
                        if (structArray[i].var == 1) //値が1(True)の時のみ実行
                        {
                            //診断機能実行
                            fnDiagOn();
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        // バイト配列から構造体に変換するヘルパーメソッド
        public static T ByteArrayToStructure<T>(byte[] bytes) where T : struct
        {
            GCHandle handle = GCHandle.Alloc(bytes, GCHandleType.Pinned);
            try
            {
                return Marshal.PtrToStructure<T>(handle.AddrOfPinnedObject());
            }
            finally
            {
                handle.Free();
            }
        }
        /// <summary>
        /// サービスメッセージハンドラ
        /// </summary>
        /// <param name="senderName"></param>
        /// <param name="messNo"></param>
        /// <param name="paramArray"></param>
        /// <returns></returns>
        public int MsgHandler(string senderName, int messNo, byte[] paramArray)
        {
            try
            {
                // 届いたEdgeメッセージに合わせて処理分岐しますよ
                switch (messNo)
                {
                    case (int)EGDEFINE.EGMSG.EM_SERVICE_RUN:            // サービス開始指令
                        // 初期化完了済みの場合のみ、処理実行します
                        if (CheckInitEnd())
                            StartService();            // サービス開始実行
                        else
                            SetInitStatus(CDefine.INIT_STS_INITRUN);    // 初期化中(RUN受信済み)に更新 ※初期化完了後にサービス開始されます
                        break;
                    case (int)EGDEFINE.EGMSG.EM_SERVICE_STOP:           // サービス終了指令
                        //Exe終了処理
                        Cleanup();
                        break;
                    case (int)EGDEFINE.EGMSG.EM_SERVICE_PAUSE:          // サービス一時停止指令
                        //サービス停止指令
                        PauseService();
                        break;
                    case (int)EGDEFINE.EGMSG.EM_SERVICE_UPDATE:			// サービスアップデート指令 本コンテナ無効
                        break;
                    case (int)CDefine.EM_MAINTE_DIAG:                           // 診断機能トリガーメッセージ
                        if (CheckInitEnd()) //初期化終了後でないとNG
                            fnDiagOn();//Diag On　処理
                        break;

                    case (int)EGDEFINE.EGMSG.EM_TAGTRIGGER_NOTIFICATION://タグトリガーメッセージ
                        //何かしら来た場合トリガー有りと判断
                        if (CheckInitEnd()) //初期化終了後でないとNG
                            fnTagTriggerOn(paramArray);//TagTrigger 処理
                        break;
                    default:
                        return -1;
                }
                return 0;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        ///*****************************************************************************
        /// <summary>
        /// サービス一時停止 ※入出力されなくなります
        /// 必要が無ければ変更不要
        /// </summary>
        /// <returns></returns>
        ///*****************************************************************************
        public int PauseService()
        {
            try
            {
                ServiceIndRun.Value = false;                  // サービス停止状態に更新
                
                return 0;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// edge ファイナライズ処理
        /// </summary>
        public void EdgeFinalize()
        {
            // Liveカウンタタイマのストップ
            ServiceIndLiveTimer.RefleshStop();

            //診断スレッド終了処理
            if (ThreadOutInfo_Dlg != null && ThreadOutInfo_Dlg.hTrigSem != null)
                ThreadOutInfo_Dlg.hTrigSem.Dispose();
            // RT-edgeフレームワーク終了
            EGAPI.EgFinalize();
        }
        /// <summary>
        /// サービス終了時の処理
        /// </summary>
        /// <returns></returns>
        public int Cleanup()
        {
            EdgeFinalize(); //edgeファイナライズ処理

            // プロセス終了
            var myself = Process.GetCurrentProcess();
            myself.CloseMainWindow();
            return 0;
        }
        /// <summary>
        /// ログ出力
        /// </summary>
        /// <param name="stServiceName"></param>
        /// <param name="stMode"></param>
        /// <param name="stFormatMess"></param>
        /// <param name="args"></param>
        public void PrintLog(string stMode, string stFormatMess, params string[] args)
        {
            string newFormatMess = stFormatMess.Replace("%", "%%"); //%が予期しないエラーになるためエスケープする
            try
            {
                edge_FW._PrintSysLog(GetServiceName(), stMode, newFormatMess, args);

            }
            catch//ログ出力に失敗時は無視
            {
            }
        }

        //
        // 追加
        //
        ///*****************************************************************************
        /// <summary>
        /// 出力スレッド情報生成
        /// </summary>
        /// <param name="usID"></param>
        /// <returns></returns>
        ///*****************************************************************************
        public IO_TRDINFO CreateSMARTThreadInfo()
        {
            IO_TRDINFO trdInfo = new IO_TRDINFO();
            trdInfo.hTrigSem = new SemaphoreSlim(1, 1);         // S.M.A.R.T.出力スレッド用トリガセマフォ
            trdInfo.threadCTS = new CancellationTokenSource();  // スレッド終了用token指定
            return trdInfo;
        }
        
        ///*****************************************************************************
        /// <summary>
        /// S.M.A.R.T.出力/取得スレッド
        /// </summary>
        /// <param name="cancelToken"></param>
        /// <param name="trdInfo"></param>
        ///*****************************************************************************
        public async void ThreadDlgSMARTService(CancellationToken cancelToken, IO_TRDINFO trdInfo)
        {
            try
            {
                if (EGAPI.EGFW.tagRef_OUT_List.Count < 1)               //TagRefが登録されていない場合は処理しない
                {
                    PrintLog("ERR", "Stop OUT Service. Because TagRef(TagRefs_OUT) is NULL.");
                    SetErrorID(CDefine.ERR_DLG_NOTTARGETTAG);       // エラーインジケータをtrueに更新
                    return;
                }
                
                await Task.Run(async () =>
                {
                    while (true)
                    {
                        await trdInfo.hTrigSem.WaitAsync();
                        // キャンセルトークンが来たら終了
                        if (cancelToken.IsCancellationRequested)
                            break;

                        //サイクル指定がある場合のみ処理を行う
                        if ((UInt64)Cycle_SMART.Value > 0)
                        {
                            if (CheckInitEnd() == true && (bool)ServiceIndRun.Value == true) //Runがtrueの場合のみ実行
                            {
                                //プログラム実行
                                int ret = EGAPI.EgMDiagExecute_DiskInfo(ref diskExecInfo);
                                if (ret != 0)
                                    PrintLog("ERR", "Failed to get Disk Info. Status{0}.", ret.ToString("X"));

                                //ファイル読み込み 全情報取得

                                ThreadCnt.Value = ++(trdInfo.uiOpeCnt);          //スレッド処理回数をTagへ書き込む
                            }
                            Thread.Sleep(TimeSpan.FromMinutes((UInt64)Cycle_SMART.Value));//秒でSleep
                            trdInfo.hTrigSem.Release();//終わったらリリースする事で実現
                        }
                    }
                });
            }
            catch (Exception ex)
            {
                PrintLog("ERR", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
    }
    /// <summary>
    /// .Live インジケータ更新用のタイマ
    /// </summary>
    public class ServiceLiveIndicatorTimer
    {
        bool IsStartF = false;
        EdgeService ServiceBody;
        System.Timers.Timer timer;

        public ServiceLiveIndicatorTimer(EdgeService _body, double n)
        {
            ServiceBody = _body;
            ServiceBody.ServiceIndLive.Value = (UInt32)0;
            timer = new System.Timers.Timer(n);
            timer.Elapsed += OnTimedEvent;
        }

        /// <summary>
        /// Liveインジケータの更新（タイマイベント）
        /// </summary>
        /// <param name="source"></param>
        /// <param name="e"></param>
        private void OnTimedEvent(Object source, System.Timers.ElapsedEventArgs e)
        {
            try
            {
                // インジケータ更新
                timer.Stop();
                ServiceBody.ServiceIndLive.Value = (UInt32)ServiceBody.ServiceIndLive.Value + (UInt32)1;

                // コンテナのステータスインジケータを更新、何かしらエラーがあれば .Error=true にします
                if ((UInt32)ServiceBody.ErrorID.Value != (UInt32)0)
                    ServiceBody.ServiceIndError.Value = (bool)true;
                else
                    ServiceBody.ServiceIndError.Value = (bool)false;
            }
            finally
            {
                // 次のタイマーイベントを許可
                timer.Start();
            }
        }
        object RefleshLock = new object();
        /// <summary>
        /// Liveインジケータの更新を開始
        /// </summary>
        public void RefleshStart()
        {
            lock (RefleshLock)  //1回だけスタートさせる
            {
                if (IsStartF == false)
                {
                    timer.Start();
                    IsStartF = true;
                }
            }
        }

        /// <summary>
        /// Liveインジケータの更新を停止
        /// </summary>
        public void RefleshStop()
        {
            lock (RefleshLock)  //1回だけスタートさせる
            {
                timer.Stop();
                IsStartF = false;
            }
        }

        
    }
}
