﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Windows;
using EgMaint;
using Microsoft.Win32;

namespace EgMGather
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        EdgeService EGS = EdgeService.GetInstance();        //RT-edgeのコード
        public bool IsInitEndFlag = false;                  //Initilizeが終了したかのフラグ

        public MainWindow()
        {
            ChangeUI();             //ウィンドウを非表示にする
            InitializeComponent();
            EGS.Startup(this);      //Edge処理初期化 内部でEgInitを呼ぶ
            EGS.PrintLog("OPE", "保守コンテナ(収集機能)は動作を開始しました。");
            //シャットダウンやログオフ時のイベント用
            SystemEvents.SessionEnding += new SessionEndingEventHandler(SystemEvents_SessionEnding);

        }
        /// <summary>
        /// Windowの表示設定(表示しない)
        /// </summary>
        private void ChangeUI()
        {
            try
            {
                this.ShowInTaskbar = false;//タスクバーに表示しない
                this.WindowState = WindowState.Minimized;//最小化
                this.Hide();
            }
            catch
            { }
        }
        /// <summary>
        /// 終了処理
        /// </summary>
        private void FinalizeMember()
        {
            try
            {
                EGS.PrintLog("DBG", "ファイル名:" + CommonFunc.GetFileName() + ",Line:" + CommonFunc.GetLine() + ",FinalizeMember開始");

                //CancelExecuteTask();                    //Executeタスク終了
                EGS.EdgeFinalize();                       //edge終了

                EGS.PrintLog("DBG", "ファイル名:" + CommonFunc.GetFileName() + ",Line:" + CommonFunc.GetLine() + ",FinalizeMember終了");
            }
            catch (Exception ex)
            {
                EGS.SetErrorID(CDefine.ERR_TRY_CATCH);//エラーコード設定
                EGS.PrintLog("ERR", "ファイル名:" + CommonFunc.GetFileName() + ",Line:" + CommonFunc.GetLine() + ",Try-catch,関数名:" + System.Reflection.MethodBase.GetCurrentMethod().Name + ",メッセージ内容:" + ex.Message);
            }
        }
        //ログオフ、シャットダウンしようとしているとき
        private void SystemEvents_SessionEnding(object sender, SessionEndingEventArgs e)
        {
            FinalizeMember();
        }
    }
}
