﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.CompilerServices;
using System.Diagnostics;
using System.IO;
using System.IO.Compression;

namespace MtnGather
{
    /// <summary>
    /// 実行クラス
    /// </summary>
    public class Executer
    {
        string m_srProcPath = "";                   // 起動プロセスパス
        string m_srCurrentPath = "";                // 起動プロセスのカレントフォルダパス
        private Encoding m_clEncoding = Encoding.UTF8;/// 文字コード

        string M_DPATH = "-d=";                     // 引数 出力フォルダパス
        string M_FNAME = "-f=";                     // 引数 出力ファイル名(拡張子無) 
        string M_SPATH = "-s=";                     // 引数 リスト記載ファイルパス　　　   
        string M_OMODE  = "-o";                     // 引数 出力ファイル名(or zip名)が既に存在する場合、上書きする

        LogSave _logSave;                           // ログを吐き出すクラス

        string m_stDir = "";                        // 出力フォルダパス デフォルトカレント
        string m_stWorkDir = "";                    // ワーク用出力フォルダ名(zip名カット) 
        string m_stFile = "";                       // 出力ファイル名(zip名) 
        string m_srZipList = "";                    // リストファイル名 デフォルト カレント/MtnGather.lst
        bool m_bOverwrite = false;                  // 既に出力先にファイル/ﾌｫﾙﾀﾞが存在する場合上書きするか否か デフォルトしない
        bool m_boPause = false;                      // 一時停止機能有効化 
        List<string> _GatherFiletList = new List<string>(); //コピーファイルパスリスト
        string LOG_NAME = "MtnGather";    // ログファイル名
        ushort _LogLevel = 7;           //ログレベル デフォルト ERR/INFO/CRT 込

        /// <summary>
        /// アーカイブ対象アイテム
        /// </summary>
        private class ZipArchiveItem
        {
            /// <summary>
            /// アーカイブ元パス
            /// </summary>
            public string m_srSrc = "";

            /// <summary>
            /// コピー先パス
            /// </summary>
            public string m_srDst = "";

            /// <summary>
            /// コピー結果
            /// </summary>
            public bool m_boResult = false;

            /// <summary>
            /// コンストラクタ
            /// </summary>
            public ZipArchiveItem()
            {

            }
        }
        /// <summary>
        /// アーカイブ対象リスト
        /// </summary>
        private List<ZipArchiveItem> m_lsZipArchive = new List<ZipArchiveItem>();



        public Executer(string[] args)
        {
            
            m_srProcPath = Process.GetCurrentProcess().MainModule.FileName; // 起動プロセスパスの取得
            m_srCurrentPath = System.AppDomain.CurrentDomain.BaseDirectory; // 起動プロセスのカレントフォルダパス
            _logSave = LogSave.GetInstance();                               // ログ出力インスタンス取得
            _logSave.Init(LOG_NAME, "./", _LogLevel, 1000);                 // ログ出力初期化
            GetArg(args);                                                   // 引数取得
        }
        public int GetArg(string[] args)
        {
            try
            {
                //引数がある場合のみ実行
                if (args.Length > 0)
                {
                    //文字列解析
                    for (int i = 0; i < args.Length; i++)
                    {

                        string arg = args[i];
                        if (arg.Contains(M_DPATH))         // -d= 出力フォルダパス指定
                        {
                            m_stDir = arg.Substring(M_DPATH.Length);
                        }
                        else if (arg.Contains(M_FNAME))    // -f= 出力ファイル名(拡張子無) 
                        {
                            m_stFile = arg.Substring(M_FNAME.Length);
                            m_stWorkDir = Path.GetFileNameWithoutExtension(m_stFile);
                        }
                        else if (arg.Contains(M_SPATH))    // -s= リスト記載ファイルパス
                        {
                            m_srZipList = arg.Substring(M_SPATH.Length);
                        }
                        else if (arg == M_OMODE)            // 上書きするか否か
                        {
                            m_bOverwrite = true;
                        }
                        
                    }
                }
                return 0;
            }
            catch (Exception ex)
            {
                fnPrintMsg(MsgType.Error, ex.Message);
                _logSave.LogOutput(LogSave.ERR, CDefine.ERR_TRY_CATCH, "ファイル名:" + GetFileName() + ",Line:" + GetLine() + ",Try-catch,関数名:fnInitCam,メッセージ内容:" + ex.Message);
                return -1;
            }
        }

        /// <summary>
        /// 実行関数
        /// </summary>
        /// <returns></returns>
        public void execute()
        {
            try
            {
                string srMsg = "";
                // リストファイルの判定
                if (false == fnJudgeZipList(ref m_srZipList, ref srMsg))
                {
                    // 判定結果が無効のため終了
                    fnPrintMsg(MsgType.Error, srMsg);
                    _logSave.LogOutput(LogSave.ERR, LogSave.INFO, "ファイル名:" + CommonFunc.GetFileName() + ",Line:" + CommonFunc.GetLine() + "," + srMsg);
                    return;
                }
                // リストファイルからアーカイブ対象フォルダ・ファイルを抽出する
                fnExtractZipList(m_srZipList, ref m_lsZipArchive);

                // アーカイブ対象フォルダ・ファイルがない場合は終了
                if (m_lsZipArchive.Count == 0)
                {
                    srMsg = "Error!! リストファイルがアーカイブ対象フォルダ・ファイルが設定されていません。 " + Environment.NewLine;
                    srMsg += "- list = " + m_srZipList + Environment.NewLine;
                    fnPrintMsg(MsgType.Error, srMsg);
                    _logSave.LogOutput(LogSave.ERR, LogSave.INFO, "ファイル名:" + CommonFunc.GetFileName() + ",Line:" + CommonFunc.GetLine() + "," + srMsg);
                    return;
                }

                // アーカイブファイルが存在するかの判定
                if (false == fnJudgeZipFile(m_srZipList, ref m_stDir, ref m_stFile, ref srMsg))
                {
                    // 判定結果が無効のため終了
                    fnPrintMsg(MsgType.Error, srMsg);
                    _logSave.LogOutput(LogSave.ERR, LogSave.INFO, "ファイル名:" + CommonFunc.GetFileName() + ",Line:" + CommonFunc.GetLine() + "," + srMsg);
                    return;
                }

                // ワークフォルダの生成
                if (false == fnCreateZipDir(ref m_stWorkDir, ref srMsg))
                {
                    // 生成失敗のため終了
                    fnPrintMsg(MsgType.Error, srMsg);
                    _logSave.LogOutput(LogSave.ERR, LogSave.INFO, "ファイル名:" + CommonFunc.GetFileName() + ",Line:" + CommonFunc.GetLine() + "," + srMsg);
                    return;
                }

                // アーカイブ情報の生成
                if (false == fnCreateZipData(m_stWorkDir, ref m_lsZipArchive, ref srMsg))
                {
                    // 生成失敗のため終了
                    fnPrintMsg(MsgType.Error, srMsg);
                    _logSave.LogOutput(LogSave.ERR, LogSave.INFO, "ファイル名:" + CommonFunc.GetFileName() + ",Line:" + CommonFunc.GetLine() + "," + srMsg);
                    return;
                }
                // 生成結果をワークフォルダに保存
                fnSaveZipResult(m_stDir, m_lsZipArchive);

                // ワークフォルダのzip化
                fnZip();

                // ワークフォルダの削除
                FileOpe.fnDeleteDir(m_stWorkDir);

                fnPrintMsg(MsgType.Info, srMsg, false);
            }
            catch (Exception ex)
            {
                fnPrintMsg(MsgType.Error, ex.Message);
                _logSave.LogOutput(LogSave.ERR, CDefine.ERR_TRY_CATCH, "ファイル名:" + GetFileName() + ",Line:" + GetLine() + ",Try-catch,関数名:fnInitCam,メッセージ内容:" + ex.Message);
            }
        } 
        /// <summary>
        /// リストファイルの判定
        /// </summary>
        /// <param name="srList">リストファイルパス</param>
        /// <param name="srMsg">メッセージ</param>
        /// <returns>false=無効 / true=有効</returns>
        private bool fnJudgeZipList(ref string srList, ref string srMsg)
        {
            bool boRet = false;

            // リストファイルの登録チェック
            if ("" == srList)
            {
                // デフォルトリストを生成する
                fnCreateZipDefList(ref srList);
            }
            else
            {
                // リストファイルの整合性チェック
                if (!fnIsValidPath(ref srList, ref srMsg))
                {
                    // リストファイルが無効なパスのため終了
                    return false;
                }
            }

            // リストファイルの存在チェック
            if (File.Exists(srList))
            {
                // ファイルあり ... 処理結果は有効
                boRet = true;
            }
            else
            {
                // ファイルなし ... 処理結果は無効
                srMsg = "Error!! リストファイルが存在しません。 " + Environment.NewLine;
                srMsg += "- list = " + srList + Environment.NewLine;
                boRet = false;
            }

            return boRet;
        }
        /// <summary>
        /// デフォルトリストファイルパスを生成する
        /// </summary>
        /// <param name="srList">リストファイル</param>
        /// <returns>false=生成失敗 / true=生成成功</returns>
        private bool fnCreateZipDefList(ref string srList)
        {
            // デフォルトリストファイルパスは起動プロセスパスの拡張子が".lst"へ変更したパス
            srList = Path.ChangeExtension(m_srProcPath, ".lst");

            // 生成成功
            return true;
        }

        /// <summary>
        /// パスの整合性チェック
        /// </summary>
        /// <param name="srPath">パス</param>
        /// <param name="srMsg">メッセージ</param>
        /// <returns>false=無効なパス / true=有効なパス</returns>
        protected bool fnIsValidPath(ref string srPath, ref string srMsg)
        {
            bool boRet;

            srMsg = "";

            try
            {
                // フルパスの取得
                srPath = Path.GetFullPath(srPath);
                // フルパス取得成功のため、有効なパスと判断
                boRet = true;
            }
            catch (Exception ex)
            {
                // エラーメッセージの生成
                srMsg = "Error!! 指定されたパスが無効です。" + Environment.NewLine;
                srMsg += "-path     = " + srPath + Environment.NewLine;
                srMsg += "-message  = " + ex.Message;
                // 例外発生のため、無効なパスと判断
                boRet = false;
            }

            return boRet;
        }/// <summary>
         /// リストファイルからアーカイブ対象フォルダ・ファイルを抽出する
         /// </summary>
         /// <param name="srList">リストファイル</param>
         /// <param name="lsArchive">アーカイブ対象フォルダ・ファイル</param>
        private void fnExtractZipList(string srList, ref List<ZipArchiveItem> lsArchive)
        {
            using (StreamReader sr = new StreamReader(srList, m_clEncoding))
            {
                string srLine = "";

                // EOFまで1行単位で読み込む
                while ((srLine = sr.ReadLine()) != null)
                {
                    // 空文字の場合は次の行へ
                    if ("" == srLine) { continue; }

                    // 不要な文字を取り除く
                    srLine = srLine.Trim();
                    srLine = srLine.TrimEnd(Path.DirectorySeparatorChar);

                    // 先頭が"#"の場合はコメント行と判断
                    if ("#" == srLine.Substring(0, 1)) { continue; }

                    // 読み込んだ情報をリストに追加
                    ZipArchiveItem clItem = new ZipArchiveItem();
                    clItem.m_srSrc = srLine;
                    lsArchive.Add(clItem);
                }
            }

            return;
        }
        /// <summary>
        /// アーカイブファイルの判定
        /// </summary>
        /// <param name="srList">リストファイル</param>
        /// <param name="srFile">ファイル</param>
        /// <param name="srMsg">メッセージ</param>
        /// <returns>false=無効 / true=有効</returns>
        private bool fnJudgeZipFile(string srList, ref string srDir, ref string srFile, ref string srMsg)
        {
            bool boRet = false;

            if("" == srDir)
            {
                fnCreateZipDefDir(ref srDir, ref srMsg);
            }
            // アーカイブファイルの登録チェック
            if ("" == srFile)
            {
                // デフォルトファイルをセットする
                fnCreateZipDefFile(ref srFile, ref srMsg);
            }
            srFile = System.IO.Path.Combine(srDir, srFile);
            
            // アーカイブファイルの整合性チェック
            if (!fnIsValidPath(ref srFile, ref srMsg))
            {
                // 無効なパスのため、処理は無効
                return false;
            }


            // アーカイブファイルの存在チェック
            if (File.Exists(srFile))
            {
                if (m_bOverwrite == true)          // 上書き
                {
                    // ファイルの削除
                    FileOpe.fnDeleteFile(srFile);
                }
                else
                { 
                    srMsg = "Error!! 既にアーカイブファイルが存在します。 " + Environment.NewLine;
                    srMsg += "- file = " + srFile + Environment.NewLine;
                    boRet = false;
                }
            }
            else
            {
                // ファイルなし ... 処理結果は有効
                boRet = true;
            }

            return boRet;
        }
        /// <summary>
        /// デフォルトフォルダパスを生成する
        /// </summary>
        /// <param name="srDir">フォルダパス</param>
        /// <param name="srMsg">メッセージ</param>
        /// <returns></returns>
        private bool fnCreateZipDefDir(ref string srDir, ref string srMsg)
        {
            srDir = m_srCurrentPath;
            // 生成成功
            return true;
        }
        /// <summary>
        /// デフォルトファイルパスを生成する
        /// </summary>
        /// <param name="srFile">ファイルパス</param>
        /// <param name="srMsg">メッセージ</param>
        /// <returns></returns>
        private bool fnCreateZipDefFile(ref string srFile, ref string srMsg)
        {
            // 「日時(yyyymmddhhmmss)_GATHER」のフォルダ名で作成
            // 現在時刻の取得
            DateTime dtNow = DateTime.Now;
            // 新しくファイル名を生成する
            srFile = dtNow.ToString("yyyyMMddHHmmss") + "_GATHER";
            srFile = Path.ChangeExtension(srFile, ".zip");

            // 生成成功
            return true;
        }
        /// <summary>
        /// ワークフォルダの生成
        /// </summary>
        /// <param name="srDir">フォルダ</param>
        /// <param name="srMsg">メッセージ</param>
        /// <returns>false=生成失敗 / true=生成成功</returns>
        private bool fnCreateZipDir(ref string srWorkDir, ref string srMsg)
        {
            srWorkDir = "";

            // 環境変数"TEMP"の取得
            string srTemp = Environment.GetEnvironmentVariable("TEMP");
            // 取得失敗はエラー
            if (null == srTemp)
            {
                srMsg = "Error!! 環境変数[TEMP]が登録されていません";
                return false;
            }

            // 現在時刻の取得
            DateTime dtNow = DateTime.Now;

            // ワークフォルダパス(%temp%archive_現在時刻)の生成
            srWorkDir = Path.Combine(srTemp, "archive_" + dtNow.ToString("yyyyMMddHHmmss"));

            // ワークフォルダの生成
            Directory.CreateDirectory(srWorkDir);

            // 生成成功
            return true;
        }
        /// <summary>
        /// アーカイブ情報の生成
        /// </summary>
        /// <param name="srWork">ワークフォルダ</param>
        /// <param name="lsArchive">アーカイブリスト</param>
        /// <param name="srMsg">メッセージ</param>
        /// <returns>false=生成失敗 / true=生成成功</returns>
        private bool fnCreateZipData(string srWork, ref List<ZipArchiveItem> lsArchive, ref string srMsg)
        {
            _logSave.LogOutput(LogSave.INF, LogSave.Function_StartEnd, "ファイル名:" + CommonFunc.GetFileName() + ",Line:" + CommonFunc.GetLine() + ",zipに含めるファイル収集を開始します。");
            fnPrintMsg(MsgType.Info, "zipに含めるファイル収集を開始します。", false);

            for (int i = 0; i < lsArchive.Count; i++)
            {
                // アーカイブ元パス
                string srSrcPath = "";
                // アーカイブ元検索文字列
                string srPattern = "";
                // パスタイプ
                FileOpe.PathType enType = FileOpe.PathType.None;

                // アーカイブ元パスのパスタイプの取得
                fnGetPathType(lsArchive[i].m_srSrc, ref enType, ref srSrcPath, ref srPattern);

                // パスタイプが該当なしの場合は次のアイテムへ
                if (FileOpe.PathType.None == enType)
                {
                    _logSave.LogOutput(LogSave.INF, LogSave.Function_StartEnd, "ファイル名:" + CommonFunc.GetFileName() + ",Line:" + CommonFunc.GetLine() + ",収集対象ファイルがないため、除外します。" + lsArchive[i].m_srSrc);
                    fnPrintMsg(MsgType.Warning, "収集対象ファイルがないため、除外します。" + lsArchive[i].m_srSrc, false);

                    // アーカイブ元パスが見つからない
                    lsArchive[i].m_srDst = "(not found path)";
                    // 処理結果の更新：コピー失敗
                    lsArchive[i].m_boResult = false;
                    // 次のアイテムへ
                    continue;
                }

                // アーカイブ先フォルダ名の作成
                // アーカイブ元パス":"を削除する
                lsArchive[i].m_srDst = srSrcPath.Replace(":", "");

                // コピー先パスの作成
                string srDstPath = Path.Combine(srWork, lsArchive[i].m_srDst);

                fnPrintMsg(MsgType.Info, "ファイルを収集します。" + lsArchive[i].m_srSrc, false);

                // コピー
                switch (enType)
                {
                    case FileOpe.PathType.Directory:
                        // フォルダコピー
                        FileOpe.fnCopyDir(srSrcPath, srDstPath, srPattern);
                        // 処理結果の更新：コピー成功
                        lsArchive[i].m_boResult = true;
                        break;
                    case FileOpe.PathType.File:
                        // ファイルコピー
                        FileOpe.fnCopyFile(srSrcPath, srDstPath);
                        // 処理結果の更新：コピー成功
                        lsArchive[i].m_boResult = true;
                        break;
                    default:
                        // 有り得ないケース
                        lsArchive[i].m_srDst = "(unknown path type=" + ((int)enType).ToString() + ")";
                        // 処理結果の更新：コピー失敗
                        lsArchive[i].m_boResult = true;
                        break;
                }
                _logSave.LogOutput(LogSave.INF, LogSave.Function_StartEnd, "ファイル名:" + CommonFunc.GetFileName() + ",Line:" + CommonFunc.GetLine() + ",ファイルを収集しました。,[" + srSrcPath + "]");

            }
            fnPrintMsg(MsgType.Info, "zipに含めるファイル収集が完了しました。", false);
            _logSave.LogOutput(LogSave.INF, LogSave.Function_StartEnd, "ファイル名:" + CommonFunc.GetFileName() + ",Line:" + CommonFunc.GetLine() + ",zipに含めるファイル収集が完了しました。");
            
            // 生成成功
            return true;
        }
        /// <summary>
        /// パスタイプの取得
        /// </summary>
        /// <param name="srTarget">判定対象パス</param>
        /// <param name="enType">パスタイプ</param>
        /// <param name="srPath">判定結果パス</param>
        /// <param name="srPattern">判定結果で得られた検索文字列</param>
        protected void fnGetPathType(string srTarget, ref FileOpe.PathType enType, ref string srPath, ref string srPattern)
        {
            try
            {
                var srFull = srTarget;
                string srMsg = "";

                //
                // 判定対象パスの整合性チェック
                //
                if (!fnIsValidPath(ref srFull, ref srMsg))
                {
                    // 判定対象パスにアスタリスクが含まれていなければ該当パスとして判断する
                    if (srFull.IndexOf('*') < 0)
                    {
                        // 無効なパスのため、該当パスなしと判断
                        enType = FileOpe.PathType.None;
                        // 判定結果パスには初期値をセット
                        srPath = "";
                        // 検索文字列は初期値をセット
                        srPattern = "";
                        // 処理終了
                        return;
                    }
                }

                //
                // 判定対象パスがディレクトリとして存在するか？
                //
                if (Directory.Exists(srFull))
                {
                    // ディレクトリとして存在する
                    enType = FileOpe.PathType.Directory;
                    // 判定結果パスにはフルパスをセット
                    srPath = srFull;
                    // 検索文字列は全ファイル対象をセット
                    srPattern = "*.*";
                    // 処理終了
                    return;
                }

                //
                // 判定対象パスがファイルとして存在するか？
                //
                if (File.Exists(srFull))
                {
                    // ファイルとして存在する
                    enType = FileOpe.PathType.File;
                    // 判定結果パスには判定対象パスをセット
                    srPath = srFull;
                    // 検索文字列は未使用
                    srPattern = "";
                    // 処理終了
                    return;
                }

                //
                // 判定対象パスがワイルド(*)で指定されたパスか？
                //
                // ディレクトリの取得
                var srDir = Path.GetDirectoryName(srFull);
                // ファイル名の取得
                var srFile = Path.GetFileName(srFull);
                // ディレクトリが存在するか？
                if (Directory.Exists(srDir))
                {
                    // ファイ名に"*"が設定されているか？
                    if (srFile.IndexOf('*') >= 0)
                    {
                        // ディレクトリとして存在する
                        enType = FileOpe.PathType.Directory;
                        // 判定結果パスには取得したディレクトリをセット
                        srPath = srDir;
                        // 検索文字列は取得したファイルをセット
                        srPattern = srFile;
                        // 処理終了
                        return;
                    }
                }

                // 該当のパスタイプなし
                enType = FileOpe.PathType.None;
                // 判定結果パスには初期値をセット
                srPath = "";
                // 検索文字列は初期値をセット
                srPattern = "";
                // 処理終了
                return;
            }
            catch (Exception)
            {
                // 例外が発生した場合は該当なし
                // 該当のパスタイプなし
                enType = FileOpe.PathType.None;
                // 判定結果パスには初期値をセット
                srPath = "";
                // 検索文字列は初期値をセット
                srPattern = "";
                // 処理終了
                return;
            }
        }
        /// <summary>
        /// アーカイブ結果の保存
        /// </summary>
        /// <param name="srWork">ワークフォルダ</param>
        /// <param name="lsArchive">アーカイブリスト</param>
        private void fnSaveZipResult(string srWork, List<ZipArchiveItem> clArchive)
        {
            // アーカイブ結果ファイルの作成
            string srPath = Path.Combine(srWork, "result.csv");

            using (StreamWriter sw = new StreamWriter(srPath, true, m_clEncoding))
            {
                // ファイルに出力する文字列
                string srLine = "";

                //
                // ヘッダ部の書き込み
                //
                srLine = "SourcePath,Result,DestinationPath";
                sw.WriteLine(srLine);

                //
                // データ部の書き込み
                //
                for (var i = 0; i < clArchive.Count; i++)
                {
                    srLine = "";
                    srLine += clArchive[i].m_srSrc + ",";
                    srLine += ((clArchive[i].m_boResult) ? "success" : "failure") + ",";
                    srLine += clArchive[i].m_srDst;

                    sw.WriteLine(srLine);
                }
            }

        }
        /// <summary>
        /// zipアーカイブ処理
        /// </summary>
        private void fnZip()
        {
            try
            {
                fnPrintMsg(MsgType.Info, "zip作成を開始します。完了までお待ちください。", false);
                fnPrintMsg(MsgType.Info, "- list          ... " + m_srZipList, false);
                fnPrintMsg(MsgType.Info, "- file          ... " + m_stFile, false);
                fnPrintMsg(MsgType.Info, "- directory     ... " + m_stWorkDir, false);

                // zip作成
                ZipFile.CreateFromDirectory(m_stWorkDir, m_stFile);

                fnPrintMsg(MsgType.Info, "zip作成が完了しました", false);
            }
            catch (Exception ex)
            {
                fnPrintMsg(MsgType.Error, ex.Message, false);
            }
        }
        /// <summary>
        /// コンソール出力タイプ
        /// </summary>
        public enum MsgType
        {
            /// <summary>
            /// 情報
            /// </summary>
            Info = 0,
            /// <summary>
            /// ワーニング
            /// </summary>
            Warning,
            /// <summary>
            /// エラー
            /// </summary>
            Error,
        }
        /// <summary>
        /// メッセージを表示する
        /// </summary>
        /// <param name="enType">メッセージタイプ</param>
        /// <param name="srMsg">メッセージ</param>
        /// <param name="boWait">trueキー入力待ち判定あり</param>
        protected void fnPrintMsg(MsgType enType, string srMsg, bool boWait = true)
        {
            // メッセージタイプでコンソール文字色を設定
            switch (enType)
            {
                case MsgType.Warning:
                    Console.ForegroundColor = ConsoleColor.Yellow;
                    Console.BackgroundColor = ConsoleColor.DarkGray;
                    break;
                case MsgType.Error:
                    Console.ForegroundColor = ConsoleColor.Red;
                    Console.BackgroundColor = ConsoleColor.DarkBlue;
                    break;
                default:
                    // 上記以外はデフォルト
                    break;
            }

            Console.WriteLine(srMsg);

            // 色をリセット
            Console.ResetColor();

            // キー入力待ちなしの場合は終了
            if (!boWait) { return; }

            // 一時停止無効の場合終了
            if (!m_boPause) { return; }

            // キー入力待ち
            Console.Write("press any key ... ");
            Console.ReadLine();

        }
        /// <summary>
        /// 実行中の行数取得(呼び出し元の行番号返却)
        /// </summary>
        /// <param name="line"></param>
        /// <returns></returns>
        public UInt16 GetLine([CallerLineNumber]int line = 0)
        {
            ushort uiLine;

            if (UInt16.TryParse(line.ToString(), out uiLine) == false)
            {
                uiLine = 0;
            }
            return uiLine;
        }
        /// <summary>
        /// 実行中のファイルネーム取得(呼び出し元のファイルパスを取得し、ファイルネームを返却)
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public string GetFileName([CallerFilePath]string path = "")
        {
            return System.IO.Path.GetFileName(path);
        }
    }
}
