﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Diagnostics;
using System.Threading;

namespace MtnReset
{
    /// <summary>
    /// 実行クラス
    /// </summary>
    public class Executer
    {
        string M_FPATH = "-f=";                     // 引数 FailSafeScriptファイルパス
        string M_EPATH = "-e=";                     // 引数 ExecuteFileパス
        string M_MODE  = "-m=";                     // 引数 OS終了モード
        LogSave _logSave;                           // ログを吐き出すクラス
        clIniFileIo _iniFile;                       // IniファイルI/O宣言
        List<string> _FailSafeScriptList = new List<string>();//FailSafeScriptリスト
        List<string> _ExecPathList = new List<string>();//実行ファイルリスト
        List<string> _ResetPathList = new List<string>();//OSリセットExeリスト
        string LOG_NAME = "MtnReset";    // iniファイルパス定義
        ushort _LogLevel = 7;           //ログレベル デフォルト ERR/INFO/CRT 込
        string INI_FILE_PATH = "./MtnReset.ini";    // iniファイルパス定義
        bool _bIsAdmin = false;                     // 管理者権限で実行されているかどうか
        string ArgFailSafeScript = "";              // 引数でのFailSafeScript指定
        string ArgExecuteFile = "";                 // 引数でのExecuteFile指定
        string ArgReset = "/s";                        // 引数でのOS終了モード指定
        public Executer(string[] args)
        {
            _logSave = LogSave.GetInstance();
            _logSave.Init(LOG_NAME, "./", _LogLevel, 1000);
            GetArg(args);
            GetIniData();                           // iniファイル情報取得
        }
        public int GetArg(string[] args)
        {
            try
            {
                string ArgMode = "";
                //引数がある場合のみ実行
                if (args.Length > 0)
                {
                    //文字列解析
                    for (int i = 0; i < args.Length; i++)
                    {

                        string arg = args[i];
                        if (arg.Contains(M_FPATH))         // -f= の引数がある場合はFailSafeScriptファイルパス指定
                        {
                            ArgFailSafeScript = arg.Substring(M_FPATH.Length);
                        }
                        else if (arg.Contains(M_EPATH))    // -e= の引数がある場合はExecuteFileファイルパス指定
                        {
                            ArgExecuteFile = arg.Substring(M_EPATH.Length);
                        }
                        else if (arg.Contains(M_MODE))    // -m= の引数がある場合はOS終了モード指定
                        {
                            ArgMode = arg.Substring(M_EPATH.Length);
                            if (string.Compare("0", ArgMode, true) == 0)
                                ArgReset = "-";
                            else if (string.Compare("1", ArgMode, true) == 0) //シャットダウン
                                ArgReset = "/s";
                            else if (string.Compare("2", ArgMode, true) == 0) //再起動
                                ArgReset = "/r";
                            else if (string.Compare("3", ArgMode, true) == 0) //強制シャットダウン
                                ArgReset = "/s -f";
                            else if (string.Compare("4", ArgMode, true) == 0) //強制再起動
                                ArgReset = "/r -f";

                        }
                    }
                }
                return 0;
            }
            catch (Exception ex)
            {
                _logSave.LogOutput(LogSave.ERR, CDefine.ERR_TRY_CATCH, "ファイル名:" + GetFileName() + ",Line:" + GetLine() + ",Try-catch,関数名:fnInitCam,メッセージ内容:" + ex.Message);
                return -1;
            }
        }

        public int GetIniData()
        {
            try
            {
                string stFailSafeScript = "FailSafeScript";
                string stExecuteFiles = "ExecuteFiles";
                string stOSResetExe = "OSResetExe";
                _iniFile = new clIniFileIo(INI_FILE_PATH);                  //IniファイルI/O宣言
                GetInitStrList(stFailSafeScript, ref _FailSafeScriptList);  //FailSafeScriptリスト
                GetInitStrList(stExecuteFiles, ref _ExecPathList);          //実行ファイルリスト
                GetInitStrList(stOSResetExe, ref _ResetPathList);           //リセット実行ファイルリスト
                
                return 0;
            }
            catch (Exception ex)
            {
                _logSave.LogOutput(LogSave.ERR, CDefine.ERR_TRY_CATCH, "ファイル名:" + GetFileName() + ",Line:" + GetLine() + ",Try-catch,関数名:fnInitCam,メッセージ内容:" + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// 実行関数
        /// </summary>
        /// <returns></returns>
        public int execute()
        {
            _logSave.LogOutput(LogSave.DBG2, LogSave.Function_StartEnd, "ファイル名:" + CommonFunc.GetFileName() + ",Line:" + CommonFunc.GetLine() + ",execute開始");
            //
            // 管理者権限で実行されているかどうか
            //
            _bIsAdmin = IsAdministrator();
            //
            // フェイルセーフスクリプト実行 ただし引数を優先
            //
            if (ArgFailSafeScript.Length > 0)
                execExecuteFile(ArgFailSafeScript);
            else//ini実行
            {
                if (_FailSafeScriptList.Count > 0)
                    execExecuteFiles(_FailSafeScriptList);
            }
            //
            // iniに記載のexe実行 ただし引数を優先
            //
            if (ArgExecuteFile.Length > 0)
                execExecuteFile(ArgExecuteFile);
            else
            {
                if (_ExecPathList.Count > 0)
                    execExecuteFiles(_ExecPathList);
            }
            //
            // iniに記載のOS操作exe実行 ただし引数を優先
            //
            if (ArgReset.Length > 0) //引数実行
            {
                if (string.Compare("-", ArgReset, true) != 0) //何も実行しないモード以外は実行する
                    execOSReset(ArgReset);
            }
            else //ini記載 exe実行
            {
                if (_ResetPathList.Count > 0)               //リストがある場合は実行する
                    execExecuteFiles(_ResetPathList);

            }
            _logSave.LogOutput(LogSave.DBG2, LogSave.Function_StartEnd, "ファイル名:" + CommonFunc.GetFileName() + ",Line:" + CommonFunc.GetLine() + ",execute終了");
            return 0;
        }
        /// <summary>
        /// 管理者権限で実行されているかどうか
        /// </summary>
        /// <returns></returns>
        public static bool IsAdministrator()
        {
            System.Security.Principal.WindowsIdentity identity
                = System.Security.Principal.WindowsIdentity.GetCurrent();
            System.Security.Principal.WindowsPrincipal principal
                = new System.Security.Principal.WindowsPrincipal(identity);
            return principal.IsInRole(System.Security.Principal.WindowsBuiltInRole.Administrator);
        }
        /// <summary>
        /// iniからリストを取得
        /// </summary>
        public void GetInitStrList(string srSectionName,ref List<string> _List)
        {
            try
            {
                long i;
                int inNo;

                inNo = _iniFile.fnIniStr(srSectionName, "Num", 0); //iniファイルから取得

                //各Noのファイル名を取得する
                for (i = 1; i <= inNo; i++)
                {
                    string tempStr = _iniFile.fnIniStr(srSectionName, i.ToString(), "");

                    // ファイルパスとして認識できる
                    if(tempStr.Length > 0)
                        _List.Add(tempStr);
                }
            }
            catch (Exception ex)
            {
                _logSave.LogOutput(LogSave.ERR, CDefine.ERR_TRY_CATCH, "ファイル名:" + GetFileName() + ",Line:" + GetLine() + ",Try-catch,関数名:fnInitCam,メッセージ内容:" + ex.Message);
            }
        }
        /// <summary>
        /// OSのシャットダウン or リスタート
        /// </summary>
        /// <param name="arg"></param>
        public void execOSReset(string arg)
        {
            try
            {
                _logSave.LogOutput(LogSave.DBG2, LogSave.Function_StartEnd, "ファイル名:" + CommonFunc.GetFileName() + ",Line:" + CommonFunc.GetLine() + ",execExecuteFile開始");

                var psi = new ProcessStartInfo
                {
                    FileName = "shutdown.exe",
                    // For Test //FileName = "MiyaShutdown.exe",
                    // コマンドライン引数を指定
                    Arguments = arg,        //引数を指定
                    UseShellExecute = false,// シェル機能を使用しない
                    CreateNoWindow = false // コンソール・ウィンドウを開く
                };
                // 開始
                var process = Process.Start(psi);
                process.WaitForExit();    //終わるまで待つ

                _logSave.LogOutput(LogSave.DBG2, LogSave.Function_StartEnd, "ファイル名:" + CommonFunc.GetFileName() + ",Line:" + CommonFunc.GetLine() + ",execExecuteFile終了");
            }
            catch (Exception ex)
            {
                _logSave.LogOutput(LogSave.ERR, CDefine.ERR_TRY_CATCH, "ファイル名:" + GetFileName() + ",Line:" + GetLine() + ",Try-catch,関数名:fnInitCam,メッセージ内容:" + ex.Message + " 実行ファイル名 : shutdown.exe");
            }
        }
        /// <summary>
        /// 実行モジュール実行
        /// </summary>
        public void execExecuteFile(string path)
        {
            try
            {
                _logSave.LogOutput(LogSave.DBG2, LogSave.Function_StartEnd, "ファイル名:" + CommonFunc.GetFileName() + ",Line:" + CommonFunc.GetLine() + ",execExecuteFile開始");

                Process p = new Process();

                p.StartInfo.FileName = path;
                if (_bIsAdmin == true) //自身も管理者権限で実行されている場合はbatも管理者で実行する
                    p.StartInfo.Verb = "RunAs"; //管理者として実行する場合

                p.Start();
                p.WaitForExit();    //終わるまで待つ
                _logSave.LogOutput(LogSave.DBG2, LogSave.Function_StartEnd, "ファイル名:" + CommonFunc.GetFileName() + ",Line:" + CommonFunc.GetLine() + ",execExecuteFile終了");
            }
            catch (Exception ex)
            {
                _logSave.LogOutput(LogSave.ERR, CDefine.ERR_TRY_CATCH, "ファイル名:" + GetFileName() + ",Line:" + GetLine() + ",Try-catch,関数名:fnInitCam,メッセージ内容:" + ex.Message + " 実行ファイル名 : " + path);
            }
        }
        /// <summary>
        /// 実行モジュール実行
        /// </summary>
        public void execExecuteFiles(List<string> List)
        {
            try
            {
                _logSave.LogOutput(LogSave.DBG2, LogSave.Function_StartEnd, "ファイル名:" + CommonFunc.GetFileName() + ",Line:" + CommonFunc.GetLine() + ",execExecuteFiles開始");
                ///実行モジュール定義数分実行
                for (int i = 0; i < List.Count(); i++)
                {
                    string fname = List[i];
                    execExecuteFile(fname);//実行 終わるまで待つ
                    Thread.Sleep(1000);
                }
                _logSave.LogOutput(LogSave.DBG2, LogSave.Function_StartEnd, "ファイル名:" + CommonFunc.GetFileName() + ",Line:" + CommonFunc.GetLine() + ",execExecuteFiles終了");

            }
            catch (Exception ex)
            {
                _logSave.LogOutput(LogSave.ERR, CDefine.ERR_TRY_CATCH, "ファイル名:" + GetFileName() + ",Line:" + GetLine() + ",Try-catch,関数名:fnInitCam,メッセージ内容:" + ex.Message);
            }
        }
        /// <summary>
        /// 実行中の行数取得(呼び出し元の行番号返却)
        /// </summary>
        /// <param name="line"></param>
        /// <returns></returns>
        public UInt16 GetLine([CallerLineNumber]int line = 0)
        {
            ushort uiLine;

            if (UInt16.TryParse(line.ToString(), out uiLine) == false)
            {
                uiLine = 0;
            }
            return uiLine;
        }
        /// <summary>
        /// 実行中のファイルネーム取得(呼び出し元のファイルパスを取得し、ファイルネームを返却)
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public string GetFileName([CallerFilePath]string path = "")
        {
            return System.IO.Path.GetFileName(path);
        }
    }
}
